@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,10 +13,10 @@
 #include <dns/messagerenderer.h>
 #include <boost/asio/ip/udp.hpp>
 #include <boost/asio/socket_base.hpp>
-#include <boost/bind.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <gtest/gtest.h>
 #include <nc_test_utils.h>
+#include <functional>
 
 using namespace std;
 using namespace isc;
@@ -29,6 +29,7 @@ using namespace isc::dns;
 using namespace isc::util;
 using namespace boost::asio;
 using namespace boost::asio::ip;
+namespace ph = std::placeholders;
 
 namespace {
 
@@ -36,8 +37,7 @@ const char* TEST_ADDRESS = "127.0.0.1";
 const uint16_t TEST_PORT = 5301;
 const size_t MAX_SIZE = 1024;
 const long TEST_TIMEOUT = 5 * 1000;
-
-// @brief Test Fixture class.
+/// @brief Test Fixture class
 //
 // This test fixture class implements DNSClient::Callback so as it can be
 // installed as a completion callback for tests it implements. This callback
@@ -64,7 +64,7 @@ public:
     int received_;
     int expected_;
 
-    // @brief Constructor.
+    /// @brief Constructor
     //
     // This constructor overrides the default logging level of asiodns logger to
     // prevent it from emitting debug messages from IOFetch class. Such an error
@@ -84,18 +84,18 @@ public:
         dns_client_.reset(new DNSClient(response_, this));
 
         // Set the test timeout to break any running tasks if they hang.
-        test_timer_.setup(boost::bind(&DNSClientTest::testTimeoutHandler, this),
+        test_timer_.setup(std::bind(&DNSClientTest::testTimeoutHandler, this),
                           TEST_TIMEOUT);
     }
 
-    // @brief Destructor.
+    /// @brief Destructor
     //
     // Sets the asiodns logging level back to DEBUG.
     virtual ~DNSClientTest() {
         asiodns::logger.setSeverity(isc::log::DEBUG);
     };
 
-    // @brief Exchange completion callback.
+    /// @brief Exchange completion callback
     //
     // This callback is called when the exchange with the DNS server is
     // complete or an error occurred. This includes the occurrence of a timeout.
@@ -133,7 +133,7 @@ public:
         }
     }
 
-    // @brief Handler invoked when test timeout is hit.
+    /// @brief Handler invoked when test timeout is hit
     //
     // This callback stops all running (hanging) tasks on IO service.
     void testTimeoutHandler() {
@@ -141,7 +141,7 @@ public:
         FAIL() << "Test timeout hit.";
     }
 
-    // @brief Handler invoked when test request is received.
+    /// @brief Handler invoked when test request is received
     //
     // This callback handler is installed when performing async read on a
     // socket to emulate reception of the DNS Update request by a server.
@@ -180,7 +180,7 @@ public:
                         *remote);
     }
 
-    // @brief Request handler for testing clients using TSIG
+    /// @brief Request handler for testing clients using TSIG
     //
     // This callback handler is installed when performing async read on a
     // socket to emulate reception of the DNS Update request with TSIG by a
@@ -219,7 +219,7 @@ public:
         dns::Message request(Message::PARSE);
         request.fromWire(received_data_buffer);
 
-        // If contex is not NULL, then we need to verify the message.
+        // If context is not NULL, then we need to verify the message.
         if (context) {
             TSIGError error = context->verify(request.getTSIGRecord(),
                                               receive_buffer_, receive_length);
@@ -312,7 +312,7 @@ public:
         /// instantly. However, it may lead to situations that the message sent
         /// in one test will not be dropped by the kernel by the time, the next
         /// test starts. This will lead to intermittent unit test errors as
-        /// described in the ticket http://kea.isc.org/ticket/3265.
+        /// described in the ticket http://oldkea.isc.org/ticket/3265.
         /// Increasing the timeout to a non-zero value mitigates this problem.
         /// The proper way to solve this problem is to receive the packet
         /// on our own and drop it. Such a fix will need to be applied not only
@@ -372,9 +372,9 @@ public:
         udp_socket.async_receive_from(boost::asio::buffer(receive_buffer_,
                                                    sizeof(receive_buffer_)),
                                       remote,
-                                      boost::bind(&DNSClientTest::udpReceiveHandler,
-                                                  this, &udp_socket, &remote, _2,
-                                                  corrupt_response));
+                                      std::bind(&DNSClientTest::udpReceiveHandler,
+                                                this, &udp_socket, &remote, ph::_2,
+                                                corrupt_response));
 
         // The socket is now ready to receive the data. Let's post some request
         // message then. Set timeout to some reasonable value to make sure that
@@ -432,10 +432,10 @@ public:
         udp_socket.async_receive_from(boost::asio::buffer(receive_buffer_,
                                                    sizeof(receive_buffer_)),
                                       remote,
-                                      boost::bind(&DNSClientTest::
-                                                  TSIGReceiveHandler, this,
-                                                  &udp_socket, &remote, _2,
-                                                  client_key, server_key));
+                                      std::bind(&DNSClientTest::
+                                                TSIGReceiveHandler, this,
+                                                &udp_socket, &remote, ph::_2,
+                                                client_key, server_key));
 
         // The socket is now ready to receive the data. Let's post some request
         // message then. Set timeout to some reasonable value to make sure that
@@ -502,7 +502,7 @@ TEST_F(DNSClientTest, runTSIGTest) {
     // Neither client nor server will attempt to sign or verify.
     runTSIGTest(nokey, nokey);
 
-    // Client signs the request, server verfies but doesn't sign.
+    // Client signs the request, server verifies but doesn't sign.
     runTSIGTest(key_one, nokey, false);
 
     // Client and server use the same key to sign and verify.
@@ -525,7 +525,7 @@ TEST_F(DNSClientTest, sendReceive) {
 
 // Verify that the DNSClient reports an error when the response is received from
 // a DNS and this response is corrupted.
-TEST_F(DNSClientTest, sendReceiveCurrupted) {
+TEST_F(DNSClientTest, sendReceiveCorrupted) {
     // true means that server's response is corrupted.
     runSendReceiveTest(true, false);
 }
