@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,14 +12,17 @@
 #include <config/command_mgr.h>
 #include <config/timeouts.h>
 #include <dhcp/dhcp4.h>
-#include <dhcp4/ctrl_dhcp4_srv.h>
-#include <dhcp4/tests/dhcp4_test_utils.h>
+#include <dhcp/libdhcp++.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/lease_mgr_factory.h>
+#include <dhcp4/ctrl_dhcp4_srv.h>
+#include <dhcp4/tests/dhcp4_test_utils.h>
 #include <hooks/hooks_manager.h>
 #include <log/logger_support.h>
 #include <stats/stats_mgr.h>
+#include <util/multi_threading_mgr.h>
+#include <util/chrono_time_utils.h>
 #include <testutils/io_utils.h>
 #include <testutils/unix_control_client.h>
 #include <testutils/sandbox.h>
@@ -50,6 +53,8 @@ using namespace isc::dhcp::test;
 using namespace isc::hooks;
 using namespace isc::stats;
 using namespace isc::test;
+using namespace isc::util;
+namespace ph = std::placeholders;
 
 namespace {
 
@@ -113,6 +118,7 @@ public:
             socket_path_ = sandbox.join("kea4.sock");
         }
         reset();
+        MultiThreadingMgr::instance().setMode(false);
     }
 
     /// @brief Destructor
@@ -125,6 +131,7 @@ public:
         CommandMgr::instance().setConnectionTimeout(TIMEOUT_DHCP_SERVER_RECEIVE_COMMAND);
 
         server_.reset();
+        MultiThreadingMgr::instance().setMode(false);
     };
 
     /// @brief Returns pointer to the server's IO service.
@@ -136,7 +143,7 @@ public:
     }
 
     void createUnixChannelServer() {
-        ::remove(socket_path_.c_str());
+        static_cast<void>(::remove(socket_path_.c_str()));
 
         // Just a simple config. The important part here is the socket
         // location information.
@@ -179,7 +186,6 @@ public:
         // changed.
         CfgMgr::instance().commit();
 
-
         ASSERT_TRUE(answer);
 
         int status = 0;
@@ -198,7 +204,7 @@ public:
     /// that no libraries are loaded and that any marker files are deleted.
     void reset() {
         // Unload any previously-loaded libraries.
-        HooksManager::unloadLibraries();
+        EXPECT_TRUE(HooksManager::unloadLibraries());
 
         // Get rid of any marker files.
         static_cast<void>(remove(LOAD_MARKER_FILE));
@@ -208,7 +214,7 @@ public:
         CfgMgr::instance().clear();
 
         // Remove unix socket file
-        ::remove(socket_path_.c_str());
+        static_cast<void>(::remove(socket_path_.c_str()));
     }
 
     /// @brief Conducts a command/response exchange via UnixCommandSocket
@@ -400,19 +406,22 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commands) {
     result = ControlledDhcpv4Srv::processCommand("shutdown", params);
     comment = parseAnswer(rcode, result);
     EXPECT_EQ(0, rcode); // expect success
+    // Exit value should default to 0.
+    EXPECT_EQ(0, server_->getExitValue());
 
-    const pid_t pid(getpid());
-    ConstElementPtr x(new isc::data::IntElement(pid));
-    params->set("pid", x);
+    // Case 3: send shutdown command with exit-value parameter.
+    ConstElementPtr x(new isc::data::IntElement(77));
+    params->set("exit-value", x);
 
-    // Case 3: send shutdown command with 1 parameter: pid
     result = ControlledDhcpv4Srv::processCommand("shutdown", params);
     comment = parseAnswer(rcode, result);
     EXPECT_EQ(0, rcode); // expect success
+
+    // Exit value should match.
+    EXPECT_EQ(77, server_->getExitValue());
 }
 
 // Check that the "libreload" command will reload libraries
-
 TEST_F(CtrlChannelDhcpv4SrvTest, libreload) {
     createUnixChannelServer();
 
@@ -427,9 +436,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, libreload) {
     HooksManager::loadLibraries(libraries);
 
     // Check they are loaded.
-    std::vector<std::string> loaded_libraries =
-        HooksManager::getLibraryNames();
-    ASSERT_TRUE(extractNames(libraries) == loaded_libraries);
+    HookLibsCollection loaded_libraries =
+        HooksManager::getLibraryInfo();
+    ASSERT_TRUE(libraries == loaded_libraries);
 
     // ... which also included checking that the marker file created by the
     // load functions exists and holds the correct value (of "12" - the
@@ -453,6 +462,54 @@ TEST_F(CtrlChannelDhcpv4SrvTest, libreload) {
     EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "1212"));
 }
 
+// Check that the "libreload" command will fail to reload libraries which are
+// not compatible when multi-threading is enabled
+TEST_F(CtrlChannelDhcpv4SrvTest, libreloadFailMultiThreading) {
+    createUnixChannelServer();
+
+    // Ensure no marker files to start with.
+    ASSERT_FALSE(checkMarkerFileExists(LOAD_MARKER_FILE));
+    ASSERT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
+
+    // Load two libraries
+    HookLibsCollection libraries;
+    libraries.push_back(make_pair(CALLOUT_LIBRARY_1, ConstElementPtr()));
+    libraries.push_back(make_pair(CALLOUT_LIBRARY_2, ConstElementPtr()));
+    HooksManager::loadLibraries(libraries);
+
+    // Check they are loaded.
+    HookLibsCollection loaded_libraries =
+        HooksManager::getLibraryInfo();
+    ASSERT_TRUE(libraries == loaded_libraries);
+
+    // ... which also included checking that the marker file created by the
+    // load functions exists and holds the correct value (of "12" - the
+    // first library appends "1" to the file, the second appends "2"). Also
+    // check that the unload marker file does not yet exist.
+    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "12"));
+    EXPECT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
+
+    // Enable multi-threading before libreload command which should now fail
+    // as the second library is not multi-threading compatible.
+    MultiThreadingMgr::instance().setMode(true);
+
+    // Now execute the "libreload" command.  This should cause the libraries
+    // to unload and to reload.
+    std::string response;
+    sendUnixCommand("{ \"command\": \"libreload\" }", response);
+    EXPECT_EQ("{ \"result\": 1, "
+              "\"text\": \"Failed to reload hooks libraries.\" }"
+              , response);
+
+    // Check that the libraries have unloaded and failed to reload.  The
+    // libraries are unloaded in the reverse order to which they are loaded.
+    // When they load, they should append information to the loading marker
+    // file.  Failing to load the second library will also unload the first
+    // library.
+    EXPECT_TRUE(checkMarkerFile(UNLOAD_MARKER_FILE, "211"));
+    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "121"));
+}
+
 // This test checks which commands are registered by the DHCPv4 server.
 TEST_F(CtrlChannelDhcpv4SrvTest, commandsRegistration) {
 
@@ -473,11 +530,13 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commandsRegistration) {
 
     EXPECT_NO_THROW(answer = CommandMgr::instance().processCommand(list_cmds));
     ASSERT_TRUE(answer);
+
     ASSERT_TRUE(answer->get("arguments"));
     std::string command_list = answer->get("arguments")->str();
 
     EXPECT_TRUE(command_list.find("\"list-commands\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"build-report\"") != string::npos);
+    EXPECT_TRUE(command_list.find("\"config-backend-pull\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"config-get\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"config-set\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"config-write\"") != string::npos);
@@ -495,6 +554,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commandsRegistration) {
     EXPECT_TRUE(command_list.find("\"statistic-sample-age-set-all\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"statistic-sample-count-set\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"statistic-sample-count-set-all\"") != string::npos);
+    EXPECT_TRUE(command_list.find("\"status-get\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"version-get\"") != string::npos);
 
     // Ok, and now delete the server. It should deregister its commands.
@@ -533,142 +593,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelShutdown) {
     EXPECT_EQ("{ \"result\": 0, \"text\": \"Shutting down.\" }",response);
 }
 
-// This test verifies that the DHCP server immediately reclaims expired
-// leases on leases-reclaim command
-TEST_F(CtrlChannelDhcpv4SrvTest, controlLeasesReclaim) {
-    createUnixChannelServer();
-
-    // Create expired leases. Leases are expired by 40 seconds ago
-    // (valid lifetime = 60, cltt = now - 100).
-    HWAddrPtr hwaddr0(new HWAddr(HWAddr::fromText("00:01:02:03:04:05")));
-    Lease4Ptr lease0(new Lease4(IOAddress("10.0.0.1"), hwaddr0,
-                                ClientIdPtr(), 60,
-                                time(NULL) - 100, SubnetID(1)));
-    HWAddrPtr hwaddr1(new HWAddr(HWAddr::fromText("01:02:03:04:05:06")));
-    Lease4Ptr lease1(new Lease4(IOAddress("10.0.0.2"), hwaddr1,
-                                ClientIdPtr(), 60,
-                                time(NULL) - 100, SubnetID(1)));
-
-    // Add leases to the database.
-    LeaseMgr& lease_mgr = LeaseMgrFactory::instance();
-    ASSERT_NO_THROW(lease_mgr.addLease(lease0));
-    ASSERT_NO_THROW(lease_mgr.addLease(lease1));
-
-    // Make sure they have been added.
-    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.1")));
-    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.2")));
-
-    // No arguments
-    std::string response;
-    sendUnixCommand("{ \"command\": \"leases-reclaim\" }", response);
-    EXPECT_EQ("{ \"result\": 1, \"text\": "
-              "\"Missing mandatory 'remove' parameter.\" }", response);
-
-    // Bad argument name
-    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
-                    "\"arguments\": { \"reclaim\": true } }", response);
-    EXPECT_EQ("{ \"result\": 1, \"text\": "
-              "\"Missing mandatory 'remove' parameter.\" }", response);
-
-    // Bad remove argument type
-    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
-                    "\"arguments\": { \"remove\": \"bogus\" } }", response);
-    EXPECT_EQ("{ \"result\": 1, \"text\": "
-              "\"'remove' parameter expected to be a boolean.\" }", response);
-
-    // Send the command
-    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
-                    "\"arguments\": { \"remove\": false } }", response);
-    EXPECT_EQ("{ \"result\": 0, \"text\": "
-              "\"Reclamation of expired leases is complete.\" }", response);
-
-    // Leases should be reclaimed, but not removed
-    ASSERT_NO_THROW(lease0 = lease_mgr.getLease4(IOAddress("10.0.0.1")));
-    ASSERT_NO_THROW(lease1 = lease_mgr.getLease4(IOAddress("10.0.0.2")));
-    ASSERT_TRUE(lease0);
-    ASSERT_TRUE(lease1);
-    EXPECT_TRUE(lease0->stateExpiredReclaimed());
-    EXPECT_TRUE(lease1->stateExpiredReclaimed());
-}
-
-// This test verifies that the DHCP server handles version-get commands
-TEST_F(CtrlChannelDhcpv4SrvTest, getversion) {
-    createUnixChannelServer();
-
-    std::string response;
-
-    // Send the version-get command
-    sendUnixCommand("{ \"command\": \"version-get\" }", response);
-    EXPECT_TRUE(response.find("\"result\": 0") != string::npos);
-    EXPECT_TRUE(response.find("log4cplus") != string::npos);
-    EXPECT_FALSE(response.find("GTEST_VERSION") != string::npos);
-
-    // Send the build-report command
-    sendUnixCommand("{ \"command\": \"build-report\" }", response);
-    EXPECT_TRUE(response.find("\"result\": 0") != string::npos);
-    EXPECT_TRUE(response.find("GTEST_VERSION") != string::npos);
-}
-
-// This test verifies that the DHCP server handles server-tag-get command
-TEST_F(CtrlChannelDhcpv4SrvTest, serverTagGet) {
-    createUnixChannelServer();
-
-    std::string response;
-    std::string expected;
-
-    // Send the server-tag-get command
-    sendUnixCommand("{ \"command\": \"server-tag-get\" }", response);
-    expected = "{ \"arguments\": { \"server-tag\": \"\" }, \"result\": 0 }";
-    EXPECT_EQ(expected, response);
-
-    // Set a value to the server tag
-    CfgMgr::instance().getCurrentCfg()->setServerTag("foobar");
-
-    // Retry...
-    sendUnixCommand("{ \"command\": \"server-tag-get\" }", response);
-    expected = "{ \"arguments\": { \"server-tag\": \"foobar\" }, \"result\": 0 }";
-}
-
-// This test verifies that the DHCP server immediately removed expired
-// This test verifies that the DHCP server immediately removed expired
-// leases on leases-reclaim command with remove = true
-TEST_F(CtrlChannelDhcpv4SrvTest, controlLeasesReclaimRemove) {
-    createUnixChannelServer();
-
-    // Create expired leases. Leases are expired by 40 seconds ago
-    // (valid lifetime = 60, cltt = now - 100).
-    HWAddrPtr hwaddr0(new HWAddr(HWAddr::fromText("00:01:02:03:04:05")));
-    Lease4Ptr lease0(new Lease4(IOAddress("10.0.0.1"), hwaddr0,
-                                ClientIdPtr(), 60,
-                                time(NULL) - 100, SubnetID(1)));
-    HWAddrPtr hwaddr1(new HWAddr(HWAddr::fromText("01:02:03:04:05:06")));
-    Lease4Ptr lease1(new Lease4(IOAddress("10.0.0.2"), hwaddr1,
-                                ClientIdPtr(), 60,
-                                time(NULL) - 100, SubnetID(1)));
-
-    // Add leases to the database.
-    LeaseMgr& lease_mgr = LeaseMgrFactory::instance();
-    ASSERT_NO_THROW(lease_mgr.addLease(lease0));
-    ASSERT_NO_THROW(lease_mgr.addLease(lease1));
-
-    // Make sure they have been added.
-    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.1")));
-    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.2")));
-
-    // Send the command
-    std::string response;
-    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
-                    "\"arguments\": { \"remove\": true } }", response);
-    EXPECT_EQ("{ \"result\": 0, \"text\": "
-              "\"Reclamation of expired leases is complete.\" }", response);
-
-    // Leases should have been removed.
-    ASSERT_NO_THROW(lease0 = lease_mgr.getLease4(IOAddress("10.0.0.1")));
-    ASSERT_NO_THROW(lease1 = lease_mgr.getLease4(IOAddress("10.0.0.2")));
-    EXPECT_FALSE(lease0);
-    EXPECT_FALSE(lease1);
-}
-
 // Tests that the server properly responds to statistics commands.  Note this
 // is really only intended to verify that the appropriate Statistics handler
 // is called based on the command.  It is not intended to be an exhaustive
@@ -686,7 +610,42 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelStats) {
     // Check statistic-get-all
     sendUnixCommand("{ \"command\" : \"statistic-get-all\", "
                     "  \"arguments\": {}}", response);
-    EXPECT_EQ("{ \"arguments\": {  }, \"result\": 0 }", response);
+
+    std::set<std::string> initial_stats = {
+        "pkt4-received",
+        "pkt4-discover-received",
+        "pkt4-offer-received",
+        "pkt4-request-received",
+        "pkt4-ack-received",
+        "pkt4-nak-received",
+        "pkt4-release-received",
+        "pkt4-decline-received",
+        "pkt4-inform-received",
+        "pkt4-unknown-received",
+        "pkt4-sent",
+        "pkt4-offer-sent",
+        "pkt4-ack-sent",
+        "pkt4-nak-sent",
+        "pkt4-parse-failed",
+        "pkt4-receive-drop"
+    };
+
+    // preparing the schema which check if all statistics are set to zero
+    std::ostringstream s;
+    s << "{ \"arguments\": { ";
+    for (auto st = initial_stats.begin(); st != initial_stats.end();) {
+        s << "\"" << *st << "\": [ [ 0, \"";
+        s << isc::util::clockToText(StatsMgr::instance().getObservation(*st)->getInteger().second);
+        s << "\" ] ]";
+        if (++st != initial_stats.end()) {
+            s << ", ";
+        }
+    }
+    s << " }, \"result\": 0 }";
+
+    auto stats_get_all = s.str();
+
+    EXPECT_EQ(stats_get_all, response);
 
     // Check statistic-reset
     sendUnixCommand("{ \"command\" : \"statistic-reset\", "
@@ -764,7 +723,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         "           \"persist\":false, \n"
         "           \"lfc-interval\": 0  \n"
         "        }, \n"
-        "       \"expired-leases-processing\": { \n"
+        "        \"expired-leases-processing\": { \n"
         "            \"reclaim-timer-wait-time\": 0, \n"
         "            \"hold-reclaimed-time\": 0, \n"
         "            \"flush-reclaimed-timer-wait-time\": 0 \n"
@@ -781,6 +740,28 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         "                \"pools\": [{ \"pool\": \"192.2.2.1-192.2.2.50\" }]}\n";
     string subnet_footer =
         "          ] \n";
+    string option_def =
+        "    ,\"option-def\": [\n"
+        "    {\n"
+        "        \"name\": \"foo\",\n"
+        "        \"code\": 163,\n"
+        "        \"type\": \"uint32\",\n"
+        "        \"array\": false,\n"
+        "        \"record-types\": \"\",\n"
+        "        \"space\": \"dhcp4\",\n"
+        "        \"encapsulate\": \"\"\n"
+        "    }\n"
+        "]\n";
+    string option_data =
+        "    ,\"option-data\": [\n"
+        "    {\n"
+        "        \"name\": \"foo\",\n"
+        "        \"code\": 163,\n"
+        "        \"space\": \"dhcp4\",\n"
+        "        \"csv-format\": true,\n"
+        "        \"data\": \"12345\"\n"
+        "    }\n"
+        "]\n";
     string control_socket_header =
         "       ,\"control-socket\": { \n"
         "       \"socket-type\": \"unix\", \n"
@@ -788,15 +769,13 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
     string control_socket_footer =
         "\"   \n} \n";
     string logger_txt =
-        "    \"Logging\": { \n"
-        "        \"loggers\": [ { \n"
+        "       ,\"loggers\": [ { \n"
         "            \"name\": \"kea\", \n"
         "            \"severity\": \"FATAL\", \n"
         "            \"output_options\": [{ \n"
         "                \"output\": \"/dev/null\" \n"
         "            }] \n"
-        "        }] \n"
-        "    } \n";
+        "        }] \n";
 
     std::ostringstream os;
 
@@ -806,12 +785,13 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         << dhcp4_cfg_txt
         << subnet1
         << subnet_footer
+        << option_def
+        << option_data
         << control_socket_header
         << socket_path_
         << control_socket_footer
-        << "}\n"                      // close dhcp4
-        << ","
         << logger_txt
+        << "}\n"                      // close dhcp4
         << "}}";
 
     // Send the config-set command
@@ -827,6 +807,10 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
     EXPECT_EQ(1, subnets->size());
 
+    OptionDefinitionPtr def =
+        LibDHCP::getRuntimeOptionDef(DHCP4_OPTION_SPACE, 163);
+    ASSERT_TRUE(def);
+
     // Create a config with malformed subnet that should fail to parse.
     os.str("");
     os << set_config_txt << ","
@@ -838,7 +822,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         << socket_path_
         << control_socket_footer
         << "}\n"                      // close dhcp4
-        "}}";
+        << "}}";
 
     // Send the config-set command
     sendUnixCommand(os.str(), response);
@@ -853,6 +837,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
     subnets = CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
     EXPECT_EQ(1, subnets->size());
 
+    def = LibDHCP::getRuntimeOptionDef(DHCP4_OPTION_SPACE, 163);
+    ASSERT_TRUE(def);
+
     // Create a valid config with two subnets and no command channel.
     // It should succeed, client should still receive the response
     os.str("");
@@ -887,41 +874,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
     CfgMgr::instance().clear();
 }
 
-// Tests that the server properly responds to shtudown command sent
-// via ControlChannel
-TEST_F(CtrlChannelDhcpv4SrvTest, listCommands) {
-    createUnixChannelServer();
-    std::string response;
-
-    sendUnixCommand("{ \"command\": \"list-commands\" }", response);
-
-    ConstElementPtr rsp;
-    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
-
-    // We expect the server to report at least the following commands:
-    checkListCommands(rsp, "build-report");
-    checkListCommands(rsp, "config-get");
-    checkListCommands(rsp, "config-reload");
-    checkListCommands(rsp, "config-set");
-    checkListCommands(rsp, "config-write");
-    checkListCommands(rsp, "list-commands");
-    checkListCommands(rsp, "leases-reclaim");
-    checkListCommands(rsp, "libreload");
-    checkListCommands(rsp, "server-tag-get");
-    checkListCommands(rsp, "shutdown");
-    checkListCommands(rsp, "statistic-get");
-    checkListCommands(rsp, "statistic-get-all");
-    checkListCommands(rsp, "statistic-remove");
-    checkListCommands(rsp, "statistic-remove-all");
-    checkListCommands(rsp, "statistic-reset");
-    checkListCommands(rsp, "statistic-reset-all");
-    checkListCommands(rsp, "statistic-sample-age-set");
-    checkListCommands(rsp, "statistic-sample-age-set-all");
-    checkListCommands(rsp, "statistic-sample-count-set");
-    checkListCommands(rsp, "statistic-sample-count-set-all");
-    checkListCommands(rsp, "version-get");
-}
-
 // Tests if the server returns its configuration using config-get.
 // Note there are separate tests that verify if toElement() called by the
 // config-get handler are actually converting the configuration correctly.
@@ -968,7 +920,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
         "           \"persist\":false, \n"
         "           \"lfc-interval\": 0  \n"
         "        }, \n"
-        "       \"expired-leases-processing\": { \n"
+        "        \"expired-leases-processing\": { \n"
         "            \"reclaim-timer-wait-time\": 0, \n"
         "            \"hold-reclaimed-time\": 0, \n"
         "            \"flush-reclaimed-timer-wait-time\": 0 \n"
@@ -992,15 +944,13 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
     string control_socket_footer =
         "\"   \n} \n";
     string logger_txt =
-        "    \"Logging\": { \n"
-        "        \"loggers\": [ { \n"
+        "       ,\"loggers\": [ { \n"
         "            \"name\": \"kea\", \n"
         "            \"severity\": \"FATAL\", \n"
         "            \"output_options\": [{ \n"
         "                \"output\": \"/dev/null\" \n"
         "            }] \n"
-        "        }] \n"
-        "    } \n";
+        "        }] \n";
 
     std::ostringstream os;
 
@@ -1013,9 +963,8 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
         << control_socket_header
         << socket_path_
         << control_socket_footer
-        << "}\n"                      // close dhcp4
-        << ","
         << logger_txt
+        << "}\n"                      // close dhcp4
         << "}}";
 
     // Send the config-set command
@@ -1042,15 +991,15 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
         << socket_path_
         << control_socket_footer
         << "}\n"                      // close dhcp4
-        "}}";
+        << "}}";
 
     // Send the config-test command
     sendUnixCommand(os.str(), response);
 
     // Should fail with a syntax error
     EXPECT_EQ("{ \"result\": 1, "
-              "\"text\": \"subnet configuration failed: mandatory 'subnet' "
-              "parameter is missing for a subnet being configured (<wire>:19:17)\" }",
+              "\"text\": \"subnet configuration failed: mandatory 'subnet' parameter "
+              "is missing for a subnet being configured (<wire>:19:17)\" }",
               response);
 
     // Check that the config was not lost
@@ -1072,7 +1021,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
     // Verify the control channel socket exists.
     ASSERT_TRUE(fileExists(socket_path_));
 
-    // Send the config-test command
+    // Send the config-test command.
     sendUnixCommand(os.str(), response);
 
     // Verify the control channel socket still exists.
@@ -1084,7 +1033,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
               "sanity checked, but not applied.\" }",
               response);
 
-    // Check that the config was not applied
+    // Check that the config was not applied.
     subnets = CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
     EXPECT_EQ(1, subnets->size());
 
@@ -1092,8 +1041,290 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
     CfgMgr::instance().clear();
 }
 
+// This test verifies that the DHCP server handles version-get commands
+TEST_F(CtrlChannelDhcpv4SrvTest, getVersion) {
+    createUnixChannelServer();
+
+    std::string response;
+
+    // Send the version-get command
+    sendUnixCommand("{ \"command\": \"version-get\" }", response);
+    EXPECT_TRUE(response.find("\"result\": 0") != string::npos);
+    EXPECT_TRUE(response.find("log4cplus") != string::npos);
+    EXPECT_FALSE(response.find("GTEST_VERSION") != string::npos);
+
+    // Send the build-report command
+    sendUnixCommand("{ \"command\": \"build-report\" }", response);
+    EXPECT_TRUE(response.find("\"result\": 0") != string::npos);
+    EXPECT_TRUE(response.find("GTEST_VERSION") != string::npos);
+}
+
+// This test verifies that the DHCP server handles server-tag-get command
+TEST_F(CtrlChannelDhcpv4SrvTest, serverTagGet) {
+    createUnixChannelServer();
+
+    std::string response;
+    std::string expected;
+
+    // Send the server-tag-get command
+    sendUnixCommand("{ \"command\": \"server-tag-get\" }", response);
+    expected = "{ \"arguments\": { \"server-tag\": \"\" }, \"result\": 0 }";
+    EXPECT_EQ(expected, response);
+
+    // Set a value to the server tag
+    CfgMgr::instance().getCurrentCfg()->setServerTag("foobar");
+
+    // Retry...
+    sendUnixCommand("{ \"command\": \"server-tag-get\" }", response);
+    expected = "{ \"arguments\": { \"server-tag\": \"foobar\" }, \"result\": 0 }";
+}
+
+// This test verifies that the DHCP server handles status-get commands
+TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
+    createUnixChannelServer();
+
+    // start_ is initialized by init.
+    ASSERT_THROW(server_->init("/no/such/file"), BadValue);
+
+    std::string response_txt;
+
+    // Send the version-get command
+    sendUnixCommand("{ \"command\": \"status-get\" }", response_txt);
+    ConstElementPtr response;
+    ASSERT_NO_THROW(response = Element::fromJSON(response_txt));
+    ASSERT_TRUE(response);
+    ASSERT_EQ(Element::map, response->getType());
+    EXPECT_EQ(2, response->size());
+    ConstElementPtr result = response->get("result");
+    ASSERT_TRUE(result);
+    ASSERT_EQ(Element::integer, result->getType());
+    EXPECT_EQ(0, result->intValue());
+    ConstElementPtr arguments = response->get("arguments");
+    ASSERT_EQ(Element::map, arguments->getType());
+
+    // The returned pid should be the pid of our process.
+    auto found_pid = arguments->get("pid");
+    ASSERT_TRUE(found_pid);
+    EXPECT_EQ(static_cast<int64_t>(getpid()), found_pid->intValue());
+
+    // It is hard to check the actual uptime (and reload) as it is based
+    // on current time. Let's just make sure it is within a reasonable
+    // range.
+    auto found_uptime = arguments->get("uptime");
+    ASSERT_TRUE(found_uptime);
+    EXPECT_LE(found_uptime->intValue(), 5);
+    EXPECT_GE(found_uptime->intValue(), 0);
+
+    auto found_reload = arguments->get("reload");
+    ASSERT_TRUE(found_reload);
+    EXPECT_LE(found_reload->intValue(), 5);
+    EXPECT_GE(found_reload->intValue(), 0);
+
+    auto found_multi_threading = arguments->get("multi-threading-enabled");
+    ASSERT_TRUE(found_multi_threading);
+    EXPECT_EQ(false, found_multi_threading->boolValue());
+
+    auto found_thread_count = arguments->get("thread-pool-size");
+    ASSERT_FALSE(found_thread_count);
+
+    auto found_queue_size = arguments->get("packet-queue-size");
+    ASSERT_FALSE(found_queue_size);
+
+    MultiThreadingMgr::instance().setMode(true);
+    MultiThreadingMgr::instance().setThreadPoolSize(4);
+    MultiThreadingMgr::instance().setPacketQueueSize(64);
+    sendUnixCommand("{ \"command\": \"status-get\" }", response_txt);
+    ASSERT_NO_THROW(response = Element::fromJSON(response_txt));
+    ASSERT_TRUE(response);
+    ASSERT_EQ(Element::map, response->getType());
+    EXPECT_EQ(2, response->size());
+    result = response->get("result");
+    ASSERT_TRUE(result);
+    ASSERT_EQ(Element::integer, result->getType());
+    EXPECT_EQ(0, result->intValue());
+    arguments = response->get("arguments");
+    ASSERT_EQ(Element::map, arguments->getType());
+
+    // The returned pid should be the pid of our process.
+    found_pid = arguments->get("pid");
+    ASSERT_TRUE(found_pid);
+    EXPECT_EQ(static_cast<int64_t>(getpid()), found_pid->intValue());
+
+    // It is hard to check the actual uptime (and reload) as it is based
+    // on current time. Let's just make sure it is within a reasonable
+    // range.
+    found_uptime = arguments->get("uptime");
+    ASSERT_TRUE(found_uptime);
+    EXPECT_LE(found_uptime->intValue(), 5);
+    EXPECT_GE(found_uptime->intValue(), 0);
+
+    found_reload = arguments->get("reload");
+    ASSERT_TRUE(found_reload);
+    EXPECT_LE(found_reload->intValue(), 5);
+    EXPECT_GE(found_reload->intValue(), 0);
+
+    found_multi_threading = arguments->get("multi-threading-enabled");
+    ASSERT_TRUE(found_multi_threading);
+    EXPECT_EQ(true, found_multi_threading->boolValue());
+
+    found_thread_count = arguments->get("thread-pool-size");
+    ASSERT_TRUE(found_thread_count);
+    EXPECT_EQ(found_thread_count->intValue(), 4);
+
+    found_queue_size = arguments->get("packet-queue-size");
+    ASSERT_TRUE(found_queue_size);
+    EXPECT_EQ(found_queue_size->intValue(), 64);
+}
+
+// This test verifies that the DHCP server handles config-backend-pull command
+TEST_F(CtrlChannelDhcpv4SrvTest, configBackendPull) {
+    createUnixChannelServer();
+
+    std::string response;
+    std::string expected;
+
+    // Send the config-backend-pull command. Note there is no configured backed.
+    sendUnixCommand("{ \"command\": \"config-backend-pull\" }", response);
+    expected = "{ \"result\": 3, \"text\": \"No config backend.\" }";
+    EXPECT_EQ(expected, response);
+}
+
+// This test verifies that the DHCP server immediately reclaims expired
+// leases on leases-reclaim command
+TEST_F(CtrlChannelDhcpv4SrvTest, controlLeasesReclaim) {
+    createUnixChannelServer();
+
+    // Create expired leases. Leases are expired by 40 seconds ago
+    // (valid lifetime = 60, cltt = now - 100).
+    HWAddrPtr hwaddr0(new HWAddr(HWAddr::fromText("00:01:02:03:04:05")));
+    Lease4Ptr lease0(new Lease4(IOAddress("10.0.0.1"), hwaddr0,
+                                ClientIdPtr(), 60,
+                                time(NULL) - 100, SubnetID(1)));
+    HWAddrPtr hwaddr1(new HWAddr(HWAddr::fromText("01:02:03:04:05:06")));
+    Lease4Ptr lease1(new Lease4(IOAddress("10.0.0.2"), hwaddr1,
+                                ClientIdPtr(), 60,
+                                time(NULL) - 100, SubnetID(1)));
+
+    // Add leases to the database.
+    LeaseMgr& lease_mgr = LeaseMgrFactory::instance();
+    ASSERT_NO_THROW(lease_mgr.addLease(lease0));
+    ASSERT_NO_THROW(lease_mgr.addLease(lease1));
+
+    // Make sure they have been added.
+    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.1")));
+    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.2")));
+
+    // No arguments
+    std::string response;
+    sendUnixCommand("{ \"command\": \"leases-reclaim\" }", response);
+    EXPECT_EQ("{ \"result\": 1, \"text\": "
+              "\"Missing mandatory 'remove' parameter.\" }", response);
+
+    // Bad argument name
+    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
+                    "\"arguments\": { \"reclaim\": true } }", response);
+    EXPECT_EQ("{ \"result\": 1, \"text\": "
+              "\"Missing mandatory 'remove' parameter.\" }", response);
+
+    // Bad remove argument type
+    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
+                    "\"arguments\": { \"remove\": \"bogus\" } }", response);
+    EXPECT_EQ("{ \"result\": 1, \"text\": "
+              "\"'remove' parameter expected to be a boolean.\" }", response);
+
+    // Send the command
+    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
+                    "\"arguments\": { \"remove\": false } }", response);
+    EXPECT_EQ("{ \"result\": 0, \"text\": "
+              "\"Reclamation of expired leases is complete.\" }", response);
+
+    // Leases should be reclaimed, but not removed
+    ASSERT_NO_THROW(lease0 = lease_mgr.getLease4(IOAddress("10.0.0.1")));
+    ASSERT_NO_THROW(lease1 = lease_mgr.getLease4(IOAddress("10.0.0.2")));
+    ASSERT_TRUE(lease0);
+    ASSERT_TRUE(lease1);
+    EXPECT_TRUE(lease0->stateExpiredReclaimed());
+    EXPECT_TRUE(lease1->stateExpiredReclaimed());
+}
+
+// This test verifies that the DHCP server immediately reclaims expired
+// leases on leases-reclaim command with remove = true
+TEST_F(CtrlChannelDhcpv4SrvTest, controlLeasesReclaimRemove) {
+    createUnixChannelServer();
+
+    // Create expired leases. Leases are expired by 40 seconds ago
+    // (valid lifetime = 60, cltt = now - 100).
+    HWAddrPtr hwaddr0(new HWAddr(HWAddr::fromText("00:01:02:03:04:05")));
+    Lease4Ptr lease0(new Lease4(IOAddress("10.0.0.1"), hwaddr0,
+                                ClientIdPtr(), 60,
+                                time(NULL) - 100, SubnetID(1)));
+    HWAddrPtr hwaddr1(new HWAddr(HWAddr::fromText("01:02:03:04:05:06")));
+    Lease4Ptr lease1(new Lease4(IOAddress("10.0.0.2"), hwaddr1,
+                                ClientIdPtr(), 60,
+                                time(NULL) - 100, SubnetID(1)));
+
+    // Add leases to the database.
+    LeaseMgr& lease_mgr = LeaseMgrFactory::instance();
+    ASSERT_NO_THROW(lease_mgr.addLease(lease0));
+    ASSERT_NO_THROW(lease_mgr.addLease(lease1));
+
+    // Make sure they have been added.
+    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.1")));
+    ASSERT_TRUE(lease_mgr.getLease4(IOAddress("10.0.0.2")));
+
+    // Send the command
+    std::string response;
+    sendUnixCommand("{ \"command\": \"leases-reclaim\", "
+                    "\"arguments\": { \"remove\": true } }", response);
+    EXPECT_EQ("{ \"result\": 0, \"text\": "
+              "\"Reclamation of expired leases is complete.\" }", response);
+
+    // Leases should have been removed.
+    ASSERT_NO_THROW(lease0 = lease_mgr.getLease4(IOAddress("10.0.0.1")));
+    ASSERT_NO_THROW(lease1 = lease_mgr.getLease4(IOAddress("10.0.0.2")));
+    ASSERT_FALSE(lease0);
+    ASSERT_FALSE(lease1);
+}
+
+// Tests that the server properly responds to shtudown command sent
+// via ControlChannel
+TEST_F(CtrlChannelDhcpv4SrvTest, listCommands) {
+    createUnixChannelServer();
+    std::string response;
+
+    sendUnixCommand("{ \"command\": \"list-commands\" }", response);
+
+    ConstElementPtr rsp;
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+
+    // We expect the server to report at least the following commands:
+    checkListCommands(rsp, "build-report");
+    checkListCommands(rsp, "config-backend-pull");
+    checkListCommands(rsp, "config-get");
+    checkListCommands(rsp, "config-reload");
+    checkListCommands(rsp, "config-set");
+    checkListCommands(rsp, "config-test");
+    checkListCommands(rsp, "config-write");
+    checkListCommands(rsp, "list-commands");
+    checkListCommands(rsp, "leases-reclaim");
+    checkListCommands(rsp, "libreload");
+    checkListCommands(rsp, "version-get");
+    checkListCommands(rsp, "server-tag-get");
+    checkListCommands(rsp, "shutdown");
+    checkListCommands(rsp, "statistic-get");
+    checkListCommands(rsp, "statistic-get-all");
+    checkListCommands(rsp, "statistic-remove");
+    checkListCommands(rsp, "statistic-remove-all");
+    checkListCommands(rsp, "statistic-reset");
+    checkListCommands(rsp, "statistic-reset-all");
+    checkListCommands(rsp, "statistic-sample-age-set");
+    checkListCommands(rsp, "statistic-sample-age-set-all");
+    checkListCommands(rsp, "statistic-sample-count-set");
+    checkListCommands(rsp, "statistic-sample-count-set-all");
+}
+
 // Tests if config-write can be called without any parameters.
-TEST_F(CtrlChannelDhcpv4SrvTest, writeConfigNoFilename) {
+TEST_F(CtrlChannelDhcpv4SrvTest, configWriteNoFilename) {
     createUnixChannelServer();
     std::string response;
 
@@ -1109,7 +1340,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, writeConfigNoFilename) {
 }
 
 // Tests if config-write can be called with a valid filename as parameter.
-TEST_F(CtrlChannelDhcpv4SrvTest, writeConfigFilename) {
+TEST_F(CtrlChannelDhcpv4SrvTest, configWriteFilename) {
     createUnixChannelServer();
     std::string response;
 
@@ -1353,8 +1584,8 @@ TEST_F(CtrlChannelDhcpv4SrvTest, longCommand) {
 
     ASSERT_NO_THROW(
         CommandMgr::instance().registerCommand("foo",
-             boost::bind(&CtrlChannelDhcpv4SrvTest::longCommandHandler,
-                         command.str(), _1, _2));
+             std::bind(&CtrlChannelDhcpv4SrvTest::longCommandHandler,
+                       command.str(), ph::_1, ph::_2));
     );
 
     createUnixChannelServer();
@@ -1412,7 +1643,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, longResponse) {
     // of a desired size.
     ASSERT_NO_THROW(
         CommandMgr::instance().registerCommand("foo",
-             boost::bind(&CtrlChannelDhcpv4SrvTest::longResponseHandler, _1, _2));
+             std::bind(&CtrlChannelDhcpv4SrvTest::longResponseHandler, ph::_1, ph::_2));
     );
 
     createUnixChannelServer();
@@ -1472,7 +1703,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, longResponse) {
 }
 
 // This test verifies that the server signals timeout if the transmission
-// takes too long, after receiving a partial command.
+// takes too long, having received a partial command.
 TEST_F(CtrlChannelDhcpv4SrvTest, connectionTimeoutPartialCommand) {
     createUnixChannelServer();
 
@@ -1522,7 +1753,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, connectionTimeoutPartialCommand) {
     // Check that the server has signalled a timeout.
     EXPECT_EQ("{ \"result\": 1, \"text\": "
               "\"Connection over control channel timed out, "
-              "discarded partial command of 19 bytes\" }" , response);
+              "discarded partial command of 19 bytes\" }", response);
 }
 
 // This test verifies that the server signals timeout if the transmission
@@ -1552,6 +1783,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, connectionTimeoutNoData) {
         ASSERT_TRUE(client);
         ASSERT_TRUE(client->connectToServer(socket_path_));
 
+        // Having sent nothing let's just wait and see if Server times us out.
         // Let's wait up to 15s for the server's response. The response
         // should arrive sooner assuming that the timeout mechanism for
         // the server is working properly.
