@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -226,9 +226,11 @@ public:
     using Dhcpv4Srv::accept;
     using Dhcpv4Srv::acceptMessageType;
     using Dhcpv4Srv::selectSubnet;
+    using Dhcpv4Srv::setSendResponsesToSource;
     using Dhcpv4Srv::VENDOR_CLASS_PREFIX;
     using Dhcpv4Srv::shutdown_;
     using Dhcpv4Srv::alloc_engine_;
+    using Dhcpv4Srv::server_port_;
     using Dhcpv4Srv::client_port_;
 };
 
@@ -438,7 +440,11 @@ public:
     /// @param config String holding server configuration in JSON format.
     /// @param commit A boolean flag indicating if the new configuration
     /// should be committed (if true), or not (if false).
-    void configure(const std::string& config, const bool commit = true);
+    /// @param open_sockets  A boolean flag indicating if sockets should
+    /// be opened (if true), or not (if false).
+    void configure(const std::string& config,
+                   const bool commit = true,
+                   const bool open_sockets = true);
 
     /// @brief Configure specified DHCP server using JSON string.
     ///
@@ -446,8 +452,12 @@ public:
     /// @param srv Instance of the server to be configured.
     /// @param commit A boolean flag indicating if the new configuration
     /// should be committed (if true), or not (if false).
-    void configure(const std::string& config, NakedDhcpv4Srv& srv,
-                   const bool commit = true);
+    /// @param open_sockets  A boolean flag indicating if sockets should
+    /// be opened (if true), or not (if false).
+    void configure(const std::string& config,
+                   NakedDhcpv4Srv& srv,
+                   const bool commit = true,
+                   const bool open_sockets = true);
 
     /// @brief Configure specified DHCP server using JSON string.
     ///
