@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -22,6 +22,7 @@
 #include <dhcp/option_custom.h>
 #include <dhcp/option.h>
 #include <dhcp/iface_mgr.h>
+#include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/lease_mgr.h>
 #include <dhcpsrv/lease_mgr_factory.h>
@@ -272,6 +273,7 @@ public:
     using Dhcpv6Srv::name_change_reqs_;
     using Dhcpv6Srv::VENDOR_CLASS_PREFIX;
     using Dhcpv6Srv::initContext;
+    using Dhcpv6Srv::server_port_;
     using Dhcpv6Srv::client_port_;
 
     /// @brief packets we pretend to receive
@@ -467,6 +469,9 @@ public:
 
     // Name of a valid network interface
     std::string valid_iface_;
+
+    // Index of a valid network interface
+    uint32_t valid_ifindex_;
 };
 
 // We need to pass one reference to the Dhcp6Client, which is defined in
