@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,6 +14,7 @@
 #include <dhcp/option.h>
 #include <cfgrpt/config_report.h>
 #include <util/encode/hex.h>
+#include <asiolink/io_error.h>
 
 #include <boost/lexical_cast.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -116,12 +117,18 @@ CommandOptions::reset() {
     renew_rate_ = 0;
     release_rate_ = 0;
     report_delay_ = 0;
+    clean_report_ = false;
+    clean_report_separator_ = "";
     clients_num_ = 0;
     mac_template_.assign(mac, mac + 6);
     duid_template_.clear();
     base_.clear();
+    addr_unique_ = false;
     mac_list_file_.clear();
     mac_list_.clear();
+    giaddr_list_file_.clear();
+    giaddr_list_.clear();
+    multi_subnet_ = false;
     num_request_.clear();
     exit_wait_time_ = 0;
     period_ = 0;
@@ -222,6 +229,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
     std::ostringstream stream;
     stream << "perfdhcp";
     int num_mac_list_files = 0;
+    int num_subnet_list_files = 0;
 
     struct option long_options[] = {
         {"scenario", required_argument, 0, LONG_OPT_SCENARIO},
@@ -231,8 +239,8 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
     // In this section we collect argument values from command line
     // they will be tuned and validated elsewhere
     while((opt = getopt_long(argc, argv,
-                             "hv46A:r:t:R:b:n:p:d:D:l:P:a:L:N:M:s:iBc1"
-                             "T:X:O:o:E:S:I:x:W:w:e:f:F:g:",
+                             "huv46A:r:t:R:b:n:p:d:D:l:P:a:L:N:M:s:iBc1"
+                             "J:T:X:O:o:E:S:I:x:W:w:e:f:F:g:C:",
                              long_options, NULL)) != -1) {
         stream << " -" << static_cast<char>(opt);
         if (optarg) {
@@ -256,6 +264,10 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             }
             break;
 
+        case 'u':
+            addr_unique_ = true;
+            break;
+
         case '4':
             check(ipversion_ == 6, "IP version already set to 6");
             ipversion_ = 4;
@@ -281,6 +293,11 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             rapid_commit_ = true;
             break;
 
+        case 'C':
+            clean_report_ = true;
+            clean_report_separator_ = optarg;
+            break;
+
         case 'd':
             check(drop_time_set_ > 1,
                   "maximum number of drops already specified, "
@@ -288,7 +305,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             try {
                 drop_time_[drop_time_set_] =
                     boost::lexical_cast<double>(optarg);
-            } catch (boost::bad_lexical_cast&) {
+            } catch (const boost::bad_lexical_cast&) {
                 isc_throw(isc::InvalidParameter,
                           "value of drop time: -d<value>"
                           " must be positive number");
@@ -308,7 +325,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                 try {
                     drop_percent =
                         boost::lexical_cast<double>(drop_arg.substr(0, percent_loc));
-                } catch (boost::bad_lexical_cast&) {
+                } catch (const boost::bad_lexical_cast&) {
                     isc_throw(isc::InvalidParameter,
                               "value of drop percentage: -D<value%>"
                               " must be 0..100");
@@ -368,6 +385,13 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                                           " positive integer");
             break;
 
+        case 'J':
+            check(num_subnet_list_files >= 1, "only one -J option can be specified");
+            num_subnet_list_files++;
+            giaddr_list_file_ = std::string(optarg);
+            loadGiaddr();
+            break;
+
         case 'l':
             localname_ = std::string(optarg);
             initIsInterface();
@@ -394,7 +418,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             break;
 
         case 'M':
-            check(num_mac_list_files >= 1, "only -M option can be specified");
+            check(num_mac_list_files >= 1, "only one -M option can be specified");
             num_mac_list_files++;
             mac_list_file_ = std::string(optarg);
             loadMacs();
@@ -450,7 +474,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             try {
                 code = boost::lexical_cast<int>(opt_text.substr(0,coma_loc));
                 check(code <= 0, "Option code can't be negative");
-            } catch (boost::bad_lexical_cast&) {
+            } catch (const boost::bad_lexical_cast&) {
                 isc_throw(InvalidParameter, "Invalid option code specified for "
                           "-o option, expected format: -o<integer>,<hexstring>");
             }
@@ -460,7 +484,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             std::vector<uint8_t> bin;
             try {
                 isc::util::encode::decodeHex(opt_text, bin);
-            } catch (BadValue& e) {
+            } catch (const BadValue& e) {
                 isc_throw(InvalidParameter, "Error during encoding option -o:"
                           << e.what());
             }
@@ -605,19 +629,20 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             server_name_ = ALL_DHCP_SERVERS;
         }
     }
+    if (!getCleanReport()) {
+        if (print_cmd_line) {
+            std::cout << "Running: " << stream.str() << std::endl;
+        }
 
-    if (print_cmd_line) {
-        std::cout << "Running: " << stream.str() << std::endl;
-    }
+        if (scenario_ == Scenario::BASIC) {
+            std::cout << "Scenario: basic." << std::endl;
+        } else if (scenario_ == Scenario::AVALANCHE) {
+            std::cout << "Scenario: avalanche." << std::endl;
+        }
 
-    if (scenario_ == Scenario::BASIC) {
-        std::cout << "Scenario: basic." << std::endl;
-    } else if (scenario_ == Scenario::AVALANCHE) {
-        std::cout << "Scenario: avalanche." << std::endl;
-    }
-
-    if (!isSingleThreaded()) {
-        std::cout << "Multi-thread mode enabled." << std::endl;
+        if (!isSingleThreaded()) {
+            std::cout << "Multi-thread mode enabled." << std::endl;
+        }
     }
 
     // Handle the local '-l' address/interface
@@ -657,7 +682,7 @@ CommandOptions::initClientsNum() {
         long long clients_num = boost::lexical_cast<long long>(optarg);
         check(clients_num < 0, errmsg);
         clients_num_ = boost::lexical_cast<uint32_t>(optarg);
-    } catch (boost::bad_lexical_cast&) {
+    } catch (const boost::bad_lexical_cast&) {
         isc_throw(isc::InvalidParameter, errmsg);
     }
 }
@@ -711,7 +736,7 @@ CommandOptions::decodeMacBase(const std::string& base) {
             try {
                 // Do actual conversion
                 ui = convertHexString(token);
-            } catch (isc::InvalidParameter&) {
+            } catch (const isc::InvalidParameter&) {
                 isc_throw(isc::InvalidParameter,
                           "invalid characters in MAC provided");
 
@@ -744,7 +769,7 @@ CommandOptions::decodeDuid(const std::string& base) {
         try {
             // Do actual conversion
             ui = convertHexString(b.substr(i, 2));
-        } catch (isc::InvalidParameter&) {
+        } catch (const isc::InvalidParameter&) {
             isc_throw(isc::InvalidParameter,
                       "invalid characters in DUID provided,"
                       " expected hex digits");
@@ -811,6 +836,31 @@ CommandOptions::convertHexString(const std::string& text) const {
     return ui;
 }
 
+bool CommandOptions::validateIP(const std::string& line) {
+    try {
+        asiolink::IOAddress ip_address_ = isc::asiolink::IOAddress(line);
+        // let's silence not used warning
+        (void) ip_address_;
+    } catch (const isc::asiolink::IOError& e) {
+        return (true);
+    }
+    giaddr_list_.push_back(line);
+    multi_subnet_ = true;
+    return (false);
+}
+
+void CommandOptions::loadGiaddr() {
+    std::string line;
+    std::ifstream infile(giaddr_list_file_.c_str());
+    size_t cnt = 0;
+    while (std::getline(infile, line)) {
+        cnt++;
+        stringstream tmp;
+        tmp << "invalid address in line: "<< cnt;
+        check(validateIP(line), tmp.str());
+    }
+}
+
 void CommandOptions::loadMacs() {
     std::string line;
     std::ifstream infile(mac_list_file_.c_str());
@@ -836,7 +886,7 @@ bool CommandOptions::decodeMacString(const std::string& line) {
       try {
         // Do actual conversion
         ui = convertHexString(token);
-      } catch (isc::InvalidParameter&) {
+      } catch (const isc::InvalidParameter&) {
         return (true);
       }
       // If conversion succeeded store byte value
@@ -957,7 +1007,7 @@ CommandOptions::positiveInteger(const std::string& errmsg) const {
         int value = boost::lexical_cast<int>(optarg);
         check(value <= 0, errmsg);
         return (value);
-    } catch (boost::bad_lexical_cast&) {
+    } catch (const boost::bad_lexical_cast&) {
         isc_throw(InvalidParameter, errmsg);
     }
 }
@@ -968,7 +1018,7 @@ CommandOptions::nonNegativeInteger(const std::string& errmsg) const {
         int value = boost::lexical_cast<int>(optarg);
         check(value < 0, errmsg);
         return (value);
-    } catch (boost::bad_lexical_cast&) {
+    } catch (const boost::bad_lexical_cast&) {
         isc_throw(InvalidParameter, errmsg);
     }
 }
@@ -1100,10 +1150,10 @@ CommandOptions::printCommandLine() const {
 void
 CommandOptions::usage() const {
     std::cout <<
-        "perfdhcp [-hv] [-4|-6] [-A<encapsulation-level>] [-e<lease-type>]"
+        "perfdhcp [-huv] [-4|-6] [-A<encapsulation-level>] [-e<lease-type>]\n"
         "         [-r<rate>] [-f<renew-rate>]\n"
-        "         [-F<release-rate>] [-t<report>] [-R<range>] [-b<base>]\n"
-        "         [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
+        "         [-F<release-rate>] [-t<report>] [-C<separator>] [-R<range>]\n"
+        "         [-b<base>] [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
         "         [-D<max-drop>] [-l<local-addr|interface>] [-P<preload>]\n"
         "         [-L<local-port>] [-N<remote-port>]\n"
         "         [-s<seed>] [-i] [-B] [-W<late-exit-delay>]\n"
@@ -1138,6 +1188,8 @@ CommandOptions::usage() const {
         "-1: Take the server-ID option from the first received message.\n"
         "-4: DHCPv4 operation (default). This is incompatible with the -6 option.\n"
         "-6: DHCPv6 operation. This is incompatible with the -4 option.\n"
+        "-u: Enable checking address uniqueness. Lease valid lifetime\n"
+        "    should not be shorter than test duration.\n"
         "-b<base>: The base mac, duid, IP, etc, used to simulate different\n"
         "    clients.  This can be specified multiple times, each instance is\n"
         "    in the <type>=<value> form, for instance:\n"
@@ -1170,6 +1222,9 @@ CommandOptions::usage() const {
         "    whether -6 is given.\n"
         "-I<ip-offset>: Offset of the (DHCPv4) IP address in the requested-IP\n"
         "    option / (DHCPv6) IA_NA option in the (second/request) template.\n"
+        "-J<giaddr-list-file>: Text file that include multiple addresses.\n"
+        "    If provided perfdhcp will choose randomly one of addresses for each\n"
+        "    exchange.\n"
         "-l<local-addr|interface>: For DHCPv4 operation, specify the local\n"
         "    hostname/address to use when communicating with the server.  By\n"
         "    default, the interface address through which traffic would\n"
@@ -1251,6 +1306,9 @@ CommandOptions::usage() const {
         "    alternative to -n, or both options can be given, in which case the\n"
         "    testing is completed when either limit is reached.\n"
         "-t<report>: Delay in seconds between two periodic reports.\n"
+        "-C<separator>: Output reduced, an argument is a separator for periodic\n"
+        "    (-t) reports generated in easy parsable mode. Data output won't be\n"
+        "    changed, remain identical as in -t option.\n"
         "\n"
         "Errors:\n"
         "- tooshort: received a too short message\n"
