@@ -1,38 +1,28 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+#include <dhcp/duid.h>
+#include <dhcp/hwaddr.h>
 #include <dhcpsrv/cfg_hosts.h>
+#include <dhcpsrv/cfg_hosts_util.h>
 #include <dhcpsrv/hosts_log.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <exceptions/exceptions.h>
+#include <util/encode/hex.h>
 #include <ostream>
+#include <string>
+#include <vector>
 
 using namespace isc::asiolink;
+using namespace isc::data;
 
 namespace isc {
 namespace dhcp {
 
-ConstHostCollection
-CfgHosts::getAll(const HWAddrPtr& hwaddr, const DuidPtr& duid) const {
-    // Do not issue logging message here because it will be logged by
-    // the getAllInternal method.
-    ConstHostCollection collection;
-    getAllInternal<ConstHostCollection>(hwaddr, duid, collection);
-    return (collection);
-}
-
-HostCollection
-CfgHosts::getAll(const HWAddrPtr& hwaddr, const DuidPtr& duid) {
-    // Do not issue logging message here because it will be logged by
-    // the getAllInternal method.
-    HostCollection collection;
-    getAllInternal<HostCollection>(hwaddr, duid, collection);
-    return (collection);
-}
-
 ConstHostCollection
 CfgHosts::getAll(const Host::IdentifierType& identifier_type,
                  const uint8_t* identifier_begin,
@@ -56,6 +46,160 @@ CfgHosts::getAll(const Host::IdentifierType& identifier_type,
     return (collection);
 }
 
+ConstHostCollection
+CfgHosts::getAll4(const SubnetID& subnet_id) const {
+    // Do not issue logging message here because it will be logged by
+    // the getAllInternal4 method.
+    ConstHostCollection collection;
+    getAllInternal4<ConstHostCollection>(subnet_id, collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getAll4(const SubnetID& subnet_id) {
+    // Do not issue logging message here because it will be logged by
+    // the getAllInternal4 method.
+    HostCollection collection;
+    getAllInternal4<HostCollection>(subnet_id, collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getAll6(const SubnetID& subnet_id) const {
+    // Do not issue logging message here because it will be logged by
+    // the getAllInternal6 method.
+    ConstHostCollection collection;
+    getAllInternal6<ConstHostCollection>(subnet_id, collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getAll6(const SubnetID& subnet_id) {
+    // Do not issue logging message here because it will be logged by
+    // the getAllInternal6 method.
+    HostCollection collection;
+    getAllInternal6<HostCollection>(subnet_id, collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getAllbyHostname(const std::string& hostname) const {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal method.
+    ConstHostCollection collection;
+    getAllbyHostnameInternal<ConstHostCollection>(hostname, collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getAllbyHostname(const std::string& hostname) {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal method.
+    HostCollection collection;
+    getAllbyHostnameInternal<HostCollection>(hostname, collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getAllbyHostname4(const std::string& hostname,
+                            const SubnetID& subnet_id) const {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal4 method.
+    ConstHostCollection collection;
+    getAllbyHostnameInternal4<ConstHostCollection>(hostname, subnet_id, collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getAllbyHostname4(const std::string& hostname,
+                            const SubnetID& subnet_id) {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal4 method.
+    HostCollection collection;
+    getAllbyHostnameInternal4<HostCollection>(hostname, subnet_id, collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getAllbyHostname6(const std::string& hostname,
+                            const SubnetID& subnet_id) const {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal6 method.
+    ConstHostCollection collection;
+    getAllbyHostnameInternal6<ConstHostCollection>(hostname, subnet_id, collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getAllbyHostname6(const std::string& hostname,
+                            const SubnetID& subnet_id) {
+    // Do not issue logging message here because it will be logged by
+    // the getAllbyHostnameInternal6 method.
+    HostCollection collection;
+    getAllbyHostnameInternal6<HostCollection>(hostname, subnet_id, collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getPage4(const SubnetID& subnet_id,
+                   size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) const {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal4 method.
+    ConstHostCollection collection;
+    getPageInternal4<ConstHostCollection>(subnet_id,
+                                          lower_host_id,
+                                          page_size,
+                                          collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getPage4(const SubnetID& subnet_id,
+                   size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal4 method.
+    HostCollection collection;
+    getPageInternal4<HostCollection>(subnet_id,
+                                     lower_host_id,
+                                     page_size,
+                                     collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getPage6(const SubnetID& subnet_id,
+                   size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) const {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal6 method.
+    ConstHostCollection collection;
+    getPageInternal6<ConstHostCollection>(subnet_id,
+                                          lower_host_id,
+                                          page_size,
+                                          collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getPage6(const SubnetID& subnet_id,
+                   size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal6 method.
+    HostCollection collection;
+    getPageInternal6<HostCollection>(subnet_id,
+                                     lower_host_id,
+                                     page_size,
+                                     collection);
+    return (collection);
+}
+
 ConstHostCollection
 CfgHosts::getAll4(const IOAddress& address) const {
     // Do not issue logging message here because it will be logged by
@@ -133,24 +277,245 @@ CfgHosts::getAllInternal(const Host::IdentifierType& identifier_type,
 
 template<typename Storage>
 void
-CfgHosts::getAllInternal(const HWAddrPtr& hwaddr, const DuidPtr& duid,
-                         Storage& storage) const {
-    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_HWADDR_DUID)
-        .arg(hwaddr ? hwaddr->toText() : "(no-hwaddr)")
-        .arg(duid ? duid->toText() : "(no-duid)");
+CfgHosts::getAllInternal4(const SubnetID& subnet_id,
+                          Storage& storage) const {
 
-    // Get hosts using HW address.
-    if (hwaddr && !hwaddr->hwaddr_.empty()) {
-        getAllInternal<Storage>(Host::IDENT_HWADDR, &hwaddr->hwaddr_[0],
-                                hwaddr->hwaddr_.size(), storage);
-    }
-    // Get hosts using DUID.
-    if (duid && !duid->getDuid().empty()) {
-        getAllInternal<Storage>(Host::IDENT_DUID, &duid->getDuid()[0],
-                                duid->getDuid().size(), storage);
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_SUBNET_ID4)
+        .arg(subnet_id);
+
+    // Use try DHCPv4 subnet id.
+    const HostContainerIndex2& idx = hosts_.get<2>();
+
+    // Append each Host object to the storage.
+    for (HostContainerIndex2::iterator host = idx.lower_bound(subnet_id);
+         host != idx.upper_bound(subnet_id);
+         ++host) {
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_SUBNET_ID4_HOST)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
     }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_SUBNET_ID4_COUNT)
+        .arg(subnet_id)
+        .arg(storage.size());
 }
 
+template<typename Storage>
+void
+CfgHosts::getAllInternal6(const SubnetID& subnet_id,
+                          Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_SUBNET_ID6)
+        .arg(subnet_id);
+
+    // Use try DHCPv6 subnet id.
+    const HostContainerIndex3& idx = hosts_.get<3>();
+
+    // Append each Host object to the storage.
+    for (HostContainerIndex3::iterator host = idx.lower_bound(subnet_id);
+         host != idx.upper_bound(subnet_id);
+         ++host) {
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_SUBNET_ID6_HOST)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_SUBNET_ID6_COUNT)
+        .arg(subnet_id)
+        .arg(storage.size());
+}
+
+template<typename Storage>
+void
+CfgHosts::getAllbyHostnameInternal(const std::string& hostname,
+                                   Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_HOSTNAME)
+        .arg(hostname);
+
+    // Use try hostname.
+    const HostContainerIndex5& idx = hosts_.get<5>();
+
+    // Append each Host object to the storage.
+    for (HostContainerIndex5::iterator host = idx.lower_bound(hostname);
+         host != idx.upper_bound(hostname);
+         ++host) {
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_HOSTNAME_HOST)
+            .arg(hostname)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_HOSTNAME_COUNT)
+        .arg(hostname)
+        .arg(storage.size());
+}
+
+template<typename Storage>
+void
+CfgHosts::getAllbyHostnameInternal4(const std::string& hostname,
+                                    const SubnetID& subnet_id,
+                                    Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE,
+              HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID4)
+        .arg(hostname)
+        .arg(subnet_id);
+
+    // Use try hostname.
+    const HostContainerIndex5& idx = hosts_.get<5>();
+
+    // Append each Host object to the storage.
+    for (HostContainerIndex5::iterator host = idx.lower_bound(hostname);
+         host != idx.upper_bound(hostname);
+         ++host) {
+        if ((*host)->getIPv4SubnetID() != subnet_id) {
+            continue;
+        }
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID4_HOST)
+            .arg(hostname)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS,
+              HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID4_COUNT)
+        .arg(hostname)
+        .arg(subnet_id)
+        .arg(storage.size());
+}
+
+template<typename Storage>
+void
+CfgHosts::getAllbyHostnameInternal6(const std::string& hostname,
+                                    const SubnetID& subnet_id,
+                                    Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE,
+              HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID6)
+        .arg(hostname)
+        .arg(subnet_id);
+
+    // Use try hostname.
+    const HostContainerIndex5& idx = hosts_.get<5>();
+
+    // Append each Host object to the storage.
+    for (HostContainerIndex5::iterator host = idx.lower_bound(hostname);
+         host != idx.upper_bound(hostname);
+         ++host) {
+        if ((*host)->getIPv6SubnetID() != subnet_id) {
+            continue;
+        }
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID6_HOST)
+            .arg(hostname)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS,
+              HOSTS_CFG_GET_ALL_HOSTNAME_SUBNET_ID6_COUNT)
+        .arg(hostname)
+        .arg(subnet_id)
+        .arg(storage.size());
+}
+
+template<typename Storage>
+void
+CfgHosts::getPageInternal4(const SubnetID& subnet_id,
+                           uint64_t lower_host_id,
+                           const HostPageSize& page_size,
+                           Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_SUBNET_ID4)
+        .arg(subnet_id);
+
+    // Use the host id last index.
+    const HostContainerIndex4& idx = hosts_.get<4>();
+    HostContainerIndex4::const_iterator host = idx.lower_bound(lower_host_id);
+
+    // Exclude the lower bound id when it is not zero.
+    if (lower_host_id &&
+        (host != idx.end()) && ((*host)->getHostId() == lower_host_id)) {
+        ++host;
+    }
+
+    // Return hosts in the subnet within the page size.
+    for (; host != idx.end(); ++host) {
+        if ((*host)->getIPv4SubnetID() != subnet_id) {
+            continue;
+        }
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_SUBNET_ID4_HOST)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+        if (storage.size() >= page_size.page_size_) {
+            break;
+        }
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_SUBNET_ID4_COUNT)
+        .arg(subnet_id)
+        .arg(storage.size());
+}
+
+template<typename Storage>
+void
+CfgHosts::getPageInternal6(const SubnetID& subnet_id,
+                           uint64_t lower_host_id,
+                           const HostPageSize& page_size,
+                           Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_SUBNET_ID6)
+        .arg(subnet_id);
+
+    // Use the host id last index.
+    const HostContainerIndex4& idx = hosts_.get<4>();
+    HostContainerIndex4::const_iterator host = idx.lower_bound(lower_host_id);
+
+    // Exclude the lower bound id when it is not zero.
+    if (lower_host_id &&
+        (host != idx.end()) && ((*host)->getHostId() == lower_host_id)) {
+        ++host;
+    }
+
+    // Return hosts in the subnet within the page size.
+    for (; host != idx.end(); ++host) {
+        if ((*host)->getIPv6SubnetID() != subnet_id) {
+            continue;
+        }
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_SUBNET_ID6_HOST)
+            .arg(subnet_id)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+        if (storage.size() >= page_size.page_size_) {
+            break;
+        }
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_SUBNET_ID6_COUNT)
+        .arg(subnet_id)
+        .arg(storage.size());
+}
+
+
 template<typename Storage>
 void
 CfgHosts::getAllInternal4(const IOAddress& address, Storage& storage) const {
@@ -209,44 +574,6 @@ CfgHosts::getAllInternal6(const IOAddress& address, Storage& storage) const {
         .arg(storage.size());
 }
 
-ConstHostPtr
-CfgHosts::get4(const SubnetID& subnet_id, const HWAddrPtr& hwaddr,
-               const DuidPtr& duid) const {
-    // Do not log here because getHostInternal logs.
-    // The false value indicates that it is an IPv4 subnet.
-    HostPtr host;
-    if (hwaddr && !hwaddr->hwaddr_.empty()) {
-        host = getHostInternal(subnet_id, false, Host::IDENT_HWADDR,
-                               &hwaddr->hwaddr_[0],
-                               hwaddr->hwaddr_.size());
-    }
-    if (!host && duid && !duid->getDuid().empty()) {
-        host = getHostInternal(subnet_id, false, Host::IDENT_DUID,
-                               &duid->getDuid()[0],
-                               duid->getDuid().size());
-    }
-    return (host);
-}
-
-HostPtr
-CfgHosts::get4(const SubnetID& subnet_id, const HWAddrPtr& hwaddr,
-               const DuidPtr& duid) {
-    // Do not log here because getHostInternal logs.
-    // The false value indicates that it is an IPv4 subnet.
-    HostPtr host;
-    if (hwaddr && !hwaddr->hwaddr_.empty()) {
-        host = getHostInternal(subnet_id, false, Host::IDENT_HWADDR,
-                               &hwaddr->hwaddr_[0],
-                               hwaddr->hwaddr_.size());
-    }
-    if (!host && duid && !duid->getDuid().empty()) {
-        host = getHostInternal(subnet_id, false, Host::IDENT_DUID,
-                               &duid->getDuid()[0],
-                               duid->getDuid().size());
-    }
-    return (host);
-}
-
 ConstHostPtr
 CfgHosts::get4(const SubnetID& subnet_id,
                const Host::IdentifierType& identifier_type,
@@ -289,46 +616,6 @@ CfgHosts::get4(const SubnetID& subnet_id, const IOAddress& address) const {
 }
 
 
-ConstHostPtr
-CfgHosts::get6(const SubnetID& subnet_id, const DuidPtr& duid,
-               const HWAddrPtr& hwaddr) const {
-    // Do not log here because getHostInternal logs.
-    // The true value indicates that it is an IPv6 subnet.
-    HostPtr host;
-    if (duid && !duid->getDuid().empty()) {
-        host = getHostInternal(subnet_id, true, Host::IDENT_DUID,
-                               &duid->getDuid()[0],
-                               duid->getDuid().size());
-    }
-    if (!host && hwaddr && !hwaddr->hwaddr_.empty()) {
-        host = getHostInternal(subnet_id, true, Host::IDENT_HWADDR,
-                               &hwaddr->hwaddr_[0],
-                               hwaddr->hwaddr_.size());
-    }
-
-    return (host);
-}
-
-HostPtr
-CfgHosts::get6(const SubnetID& subnet_id, const DuidPtr& duid,
-               const HWAddrPtr& hwaddr) {
-    // Do not log here because getHostInternal logs.
-    // The true value indicates that it is an IPv6 subnet.
-    HostPtr host;
-    if (duid && !duid->getDuid().empty()) {
-        host = getHostInternal(subnet_id, true, Host::IDENT_DUID,
-                               &duid->getDuid()[0],
-                               duid->getDuid().size());
-    }
-    if (!host && hwaddr && !hwaddr->hwaddr_.empty()) {
-        host = getHostInternal(subnet_id, true, Host::IDENT_HWADDR,
-                               &hwaddr->hwaddr_[0],
-                               hwaddr->hwaddr_.size());
-    }
-
-    return (host);
-}
-
 ConstHostPtr
 CfgHosts::get6(const SubnetID& subnet_id,
                const Host::IdentifierType& identifier_type,
@@ -554,8 +841,9 @@ CfgHosts::add(const HostPtr& host) {
                   " is added to the configuration");
     }
 
-    // At least one subnet ID must be non-zero
-    if (host->getIPv4SubnetID() == 0 && host->getIPv6SubnetID() == 0) {
+    // At least one subnet ID must be used
+    if (host->getIPv4SubnetID() == SUBNET_ID_UNUSED &&
+        host->getIPv6SubnetID() == SUBNET_ID_UNUSED) {
         isc_throw(BadValue, "must not use both IPv4 and IPv6 subnet ids of"
                   " 0 when adding new host reservation");
     }
@@ -567,12 +855,13 @@ CfgHosts::add(const HostPtr& host) {
 
 void
 CfgHosts::add4(const HostPtr& host) {
-    /// @todo This may need further sanity checks.
+
     HWAddrPtr hwaddr = host->getHWAddress();
     DuidPtr duid = host->getDuid();
 
     // There should be at least one resource reserved: hostname, IPv4
     // address, siaddr, sname, file or IPv6 address or prefix.
+    /// @todo: this check should be done in add(), not in add4()
     if (host->getHostname().empty() &&
         (host->getIPv4Reservation().isV4Zero()) &&
         !host->hasIPv6Reservation() &&
@@ -596,28 +885,46 @@ CfgHosts::add4(const HostPtr& host) {
     }
 
     // Check for duplicates for the specified IPv4 subnet.
-    if ((host->getIPv4SubnetID() > 0) &&
-        get4(host->getIPv4SubnetID(), hwaddr, duid)) {
-        isc_throw(DuplicateHost, "failed to add new host using the HW"
-                  " address '" << (hwaddr ? hwaddr->toText(false) : "(null)")
-                  << " and DUID '" << (duid ? duid->toText() : "(null)")
-                  << "' to the IPv4 subnet id '" << host->getIPv4SubnetID()
-                  << "' as this host has already been added");
-
-
+    if (host->getIPv4SubnetID() != SUBNET_ID_UNUSED) {
+        if (hwaddr && !hwaddr->hwaddr_.empty() &&
+            get4(host->getIPv4SubnetID(), Host::IDENT_HWADDR,
+                 &hwaddr->hwaddr_[0], hwaddr->hwaddr_.size())) {
+            isc_throw(DuplicateHost, "failed to add new host using the HW"
+                      << " address '" << hwaddr->toText(false)
+                      << "' to the IPv4 subnet id '" << host->getIPv4SubnetID()
+                      << "' as this host has already been added");
+        }
+        if (duid && !duid->getDuid().empty() &&
+            get4(host->getIPv4SubnetID(), Host::IDENT_DUID,
+                 &duid->getDuid()[0], duid->getDuid().size())) {
+            isc_throw(DuplicateHost, "failed to add new host using the "
+                      << "DUID '" << duid->toText()
+                      << "' to the IPv4 subnet id '" << host->getIPv4SubnetID()
+                      << "' as this host has already been added");
+        }
     // Check for duplicates for the specified IPv6 subnet.
-    } else if (host->getIPv6SubnetID() &&
-               get6(host->getIPv6SubnetID(), duid, hwaddr)) {
-        isc_throw(DuplicateHost, "failed to add new host using the HW"
-                  " address '" << (hwaddr ? hwaddr->toText(false) : "(null)")
-                  << " and DUID '" << (duid ? duid->toText() : "(null)")
-                  << "' to the IPv6 subnet id '" << host->getIPv6SubnetID()
-                  << "' as this host has already been added");
+    } else if (host->getIPv6SubnetID() != SUBNET_ID_UNUSED) {
+        if (duid && !duid->getDuid().empty() &&
+            get6(host->getIPv6SubnetID(), Host::IDENT_DUID,
+                 &duid->getDuid()[0], duid->getDuid().size())) {
+            isc_throw(DuplicateHost, "failed to add new host using the "
+                      << "DUID '" << duid->toText()
+                      << "' to the IPv6 subnet id '" << host->getIPv6SubnetID()
+                      << "' as this host has already been added");
+        }
+        if (hwaddr && !hwaddr->hwaddr_.empty() &&
+            get6(host->getIPv6SubnetID(), Host::IDENT_HWADDR,
+                 &hwaddr->hwaddr_[0], hwaddr->hwaddr_.size())) {
+            isc_throw(DuplicateHost, "failed to add new host using the HW"
+                      << " address '" << hwaddr->toText(false)
+                      << "' to the IPv6 subnet id '" << host->getIPv6SubnetID()
+                      << "' as this host has already been added");
+        }
     }
 
     // Check if the address is already reserved for the specified IPv4 subnet.
     if (!host->getIPv4Reservation().isV4Zero() &&
-        (host->getIPv4SubnetID() > 0) &&
+        (host->getIPv4SubnetID() != SUBNET_ID_UNUSED) &&
         get4(host->getIPv4SubnetID(), host->getIPv4Reservation())) {
         isc_throw(ReservedAddress, "failed to add new host using the HW"
                   " address '" << (hwaddr ? hwaddr->toText(false) : "(null)")
@@ -627,15 +934,30 @@ CfgHosts::add4(const HostPtr& host) {
                   << ": There's already a reservation for this address");
     }
 
-    /// @todo This may need further sanity checks.
+    // Check if the (identifier type, identifier) tuple is already used.
+    const std::vector<uint8_t>& id = host->getIdentifier();
+    if ((host->getIPv4SubnetID() != SUBNET_ID_UNUSED) && !id.empty()) {
+        if (get4(host->getIPv4SubnetID(), host->getIdentifierType(), &id[0],
+                 id.size())) {
+            isc_throw(DuplicateHost, "failed to add duplicate IPv4 host using identifier: "
+                      << Host::getIdentifierAsText(host->getIdentifierType(),
+                                                   &id[0], id.size()));
+        }
+    }
 
     // This is a new instance - add it.
+    host->setHostId(++next_host_id_);
     hosts_.insert(host);
 }
 
 void
 CfgHosts::add6(const HostPtr& host) {
-    /// @todo This may need further sanity checks.
+
+    if (host->getIPv6SubnetID() == SUBNET_ID_UNUSED) {
+        // This is IPv4-only host. No need to add it to v6 tables.
+        return;
+    }
+
     HWAddrPtr hwaddr = host->getHWAddress();
     DuidPtr duid = host->getDuid();
 
@@ -667,5 +989,111 @@ CfgHosts::add6(const HostPtr& host) {
     }
 }
 
+bool
+CfgHosts::del(const SubnetID& /*subnet_id*/, const asiolink::IOAddress& /*addr*/) {
+    /// @todo: Implement host removal
+    isc_throw(NotImplemented, "sorry, not implemented");
+    return (false);
+}
+
+size_t
+CfgHosts::delAll4(const SubnetID& subnet_id) {
+    HostContainerIndex2& idx = hosts_.get<2>();
+    size_t erased = idx.erase(subnet_id);
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_DEL_ALL_SUBNET4)
+        .arg(erased)
+        .arg(subnet_id);
+
+    return (erased);
+}
+
+bool
+CfgHosts::del4(const SubnetID& /*subnet_id*/,
+               const Host::IdentifierType& /*identifier_type*/,
+               const uint8_t* /*identifier_begin*/,
+               const size_t /*identifier_len*/) {
+    /// @todo: Implement host removal
+    isc_throw(NotImplemented, "sorry, not implemented");
+    return (false);
+}
+
+size_t
+CfgHosts::delAll6(const SubnetID& subnet_id) {
+    // Delete IPv6 reservations.
+    HostContainer6Index2& idx6 = hosts6_.get<2>();
+    size_t erased_addresses = idx6.erase(subnet_id);
+
+    // Delete hosts.
+    HostContainerIndex3& idx = hosts_.get<3>();
+    size_t erased_hosts = idx.erase(subnet_id);
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_DEL_ALL_SUBNET6)
+        .arg(erased_hosts)
+        .arg(erased_addresses)
+        .arg(subnet_id);
+
+    return (erased_hosts);
+}
+
+bool
+CfgHosts::del6(const SubnetID& /*subnet_id*/,
+               const Host::IdentifierType& /*identifier_type*/,
+               const uint8_t* /*identifier_begin*/,
+               const size_t /*identifier_len*/) {
+    /// @todo: Implement host removal
+    isc_throw(NotImplemented, "sorry, not implemented");
+    return (false);
+}
+
+ElementPtr
+CfgHosts::toElement() const {
+    uint16_t family = CfgMgr::instance().getFamily();
+    if (family == AF_INET) {
+        return (toElement4());
+    } else if (family == AF_INET6) {
+        return (toElement6());
+    } else {
+        isc_throw(ToElementError, "CfgHosts::toElement: unknown "
+                  "address family: " << family);
+    }
+}
+
+ElementPtr
+CfgHosts::toElement4() const {
+    CfgHostsList result;
+    // Iterate using arbitrary the index 0
+    const HostContainerIndex0& idx = hosts_.get<0>();
+    for (HostContainerIndex0::const_iterator host = idx.begin();
+         host != idx.end(); ++host) {
+
+        // Convert host to element representation
+        ElementPtr map = (*host)->toElement4();
+
+        // Push it on the list
+        SubnetID subnet_id = (*host)->getIPv4SubnetID();
+        result.add(subnet_id, map);
+    }
+    return (result.externalize());
+}
+
+ElementPtr
+CfgHosts::toElement6() const {
+    CfgHostsList result;
+    // Iterate using arbitrary the index 0
+    const HostContainerIndex0& idx = hosts_.get<0>();
+    for (HostContainerIndex0::const_iterator host = idx.begin();
+         host != idx.end(); ++host) {
+
+        // Convert host to Element representation
+        ElementPtr map = (*host)->toElement6();
+
+        // Push it on the list
+        SubnetID subnet_id = (*host)->getIPv6SubnetID();
+        result.add(subnet_id, map);
+    }
+    return (result.externalize());
+}
+
 } // end of namespace isc::dhcp
 } // end of namespace isc
