@@ -82,7 +82,8 @@ CfgMgr::clear() {
     }
     configs_.clear();
     external_configs_.clear();
-    ensureCurrentAllocated();
+    D2ClientConfigPtr d2_default_conf(new D2ClientConfig());
+    setD2ClientConfig(d2_default_conf);
 }
 
 void
@@ -107,6 +108,10 @@ CfgMgr::commit() {
         }
     }
 
+    // Set the last commit timestamp.
+    auto now = boost::posix_time::second_clock::universal_time();
+    configuration_->setLastCommitTime(now);
+
     // Now we need to set the statistics back.
     configuration_->updateStatistics();
 }
