@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -36,47 +36,57 @@ namespace dhcp {
 /// list and map types for entries.
 /// Order follows global_param rule in bison grammar.
 const SimpleKeywords SimpleParser4::GLOBAL4_PARAMETERS = {
-    { "valid-lifetime",               Element::integer },
-    { "min-valid-lifetime",           Element::integer },
-    { "max-valid-lifetime",           Element::integer },
-    { "renew-timer",                  Element::integer },
-    { "rebind-timer",                 Element::integer },
-    { "decline-probation-period",     Element::integer },
-    { "subnet4",                      Element::list },
-    { "shared-networks",              Element::list },
-    { "interfaces-config",            Element::map },
-    { "lease-database",               Element::map },
-    { "hosts-database",               Element::map },
-    { "hosts-databases",              Element::list },
-    { "host-reservation-identifiers", Element::list },
-    { "client-classes",               Element::list },
-    { "option-def",                   Element::list },
-    { "option-data",                  Element::list },
-    { "hooks-libraries",              Element::list },
-    { "expired-leases-processing",    Element::map },
-    { "dhcp4o6-port",                 Element::integer },
-    { "control-socket",               Element::map },
-    { "dhcp-queue-control",           Element::map },
-    { "dhcp-ddns",                    Element::map },
-    { "echo-client-id",               Element::boolean },
-    { "match-client-id",              Element::boolean },
-    { "authoritative",                Element::boolean },
-    { "next-server",                  Element::string },
-    { "server-hostname",              Element::string },
-    { "boot-file-name",               Element::string },
-    { "user-context",                 Element::map },
-    { "comment",                      Element::string },
-    { "sanity-checks",                Element::map },
-    { "reservations",                 Element::list },
-    { "config-control",               Element::map },
-    { "server-tag",                   Element::string },
-    { "reservation-mode",             Element::string },
-    { "calculate-tee-times",          Element::boolean },
-    { "t1-percent",                   Element::real },
-    { "t2-percent",                   Element::real },
-    { "loggers",                      Element::list },
-    { "hostname-char-set",            Element::string },
-    { "hostname-char-replacement",    Element::string }
+    { "valid-lifetime",                 Element::integer },
+    { "min-valid-lifetime",             Element::integer },
+    { "max-valid-lifetime",             Element::integer },
+    { "renew-timer",                    Element::integer },
+    { "rebind-timer",                   Element::integer },
+    { "decline-probation-period",       Element::integer },
+    { "subnet4",                        Element::list },
+    { "shared-networks",                Element::list },
+    { "interfaces-config",              Element::map },
+    { "lease-database",                 Element::map },
+    { "hosts-database",                 Element::map },
+    { "hosts-databases",                Element::list },
+    { "host-reservation-identifiers",   Element::list },
+    { "client-classes",                 Element::list },
+    { "option-def",                     Element::list },
+    { "option-data",                    Element::list },
+    { "hooks-libraries",                Element::list },
+    { "expired-leases-processing",      Element::map },
+    { "dhcp4o6-port",                   Element::integer },
+    { "control-socket",                 Element::map },
+    { "dhcp-queue-control",             Element::map },
+    { "dhcp-ddns",                      Element::map },
+    { "echo-client-id",                 Element::boolean },
+    { "match-client-id",                Element::boolean },
+    { "authoritative",                  Element::boolean },
+    { "next-server",                    Element::string },
+    { "server-hostname",                Element::string },
+    { "boot-file-name",                 Element::string },
+    { "user-context",                   Element::map },
+    { "comment",                        Element::string },
+    { "sanity-checks",                  Element::map },
+    { "reservations",                   Element::list },
+    { "config-control",                 Element::map },
+    { "server-tag",                     Element::string },
+    { "reservation-mode",               Element::string },
+    { "calculate-tee-times",            Element::boolean },
+    { "t1-percent",                     Element::real },
+    { "t2-percent",                     Element::real },
+    { "loggers",                        Element::list },
+    { "hostname-char-set",              Element::string },
+    { "hostname-char-replacement",      Element::string },
+    { "ddns-send-updates",              Element::boolean },
+    { "ddns-override-no-update",        Element::boolean },
+    { "ddns-override-client-update",    Element::boolean },
+    { "ddns-replace-client-name",       Element::string },
+    { "ddns-generated-prefix",          Element::string },
+    { "ddns-qualifying-suffix",         Element::string },
+    { "store-extended-info",            Element::boolean },
+    { "statistic-default-sample-count", Element::integer },
+    { "statistic-default-sample-age",   Element::integer },
+    { "multi-threading",                Element::map }
 };
 
 /// @brief This table defines default global values for DHCPv4
@@ -85,20 +95,31 @@ const SimpleKeywords SimpleParser4::GLOBAL4_PARAMETERS = {
 /// in Dhcp4) are optional. If not defined, the following values will be
 /// used.
 const SimpleDefaults SimpleParser4::GLOBAL4_DEFAULTS = {
-    { "valid-lifetime",            Element::integer, "7200" },
-    { "decline-probation-period",  Element::integer, "86400" }, // 24h
-    { "dhcp4o6-port",              Element::integer, "0" },
-    { "echo-client-id",            Element::boolean, "true" },
-    { "match-client-id",           Element::boolean, "true" },
-    { "authoritative",             Element::boolean, "false" },
-    { "next-server",               Element::string,  "0.0.0.0" },
-    { "server-hostname",           Element::string,  "" },
-    { "boot-file-name",            Element::string,  "" },
-    { "server-tag",                Element::string,  "" },
-    { "reservation-mode",          Element::string,  "all" },
-    { "calculate-tee-times",       Element::boolean, "false" },
-    { "t1-percent",                Element::real,    ".50" },
-    { "t2-percent",                Element::real,    ".875" }
+    { "valid-lifetime",                 Element::integer, "7200" },
+    { "decline-probation-period",       Element::integer, "86400" }, // 24h
+    { "dhcp4o6-port",                   Element::integer, "0" },
+    { "echo-client-id",                 Element::boolean, "true" },
+    { "match-client-id",                Element::boolean, "true" },
+    { "authoritative",                  Element::boolean, "false" },
+    { "next-server",                    Element::string,  "0.0.0.0" },
+    { "server-hostname",                Element::string,  "" },
+    { "boot-file-name",                 Element::string,  "" },
+    { "server-tag",                     Element::string,  "" },
+    { "reservation-mode",               Element::string,  "all" },
+    { "calculate-tee-times",            Element::boolean, "false" },
+    { "t1-percent",                     Element::real,    ".50" },
+    { "t2-percent",                     Element::real,    ".875" },
+    { "ddns-send-updates",              Element::boolean, "true" },
+    { "ddns-override-no-update",        Element::boolean, "false" },
+    { "ddns-override-client-update",    Element::boolean, "false" },
+    { "ddns-replace-client-name",       Element::string,  "never" },
+    { "ddns-generated-prefix",          Element::string,  "myhost" },
+    { "ddns-qualifying-suffix",         Element::string,  "" },
+    { "hostname-char-set",              Element::string,  "[^A-Za-z0-9.-]" },
+    { "hostname-char-replacement",      Element::string,  "" },
+    { "store-extended-info",            Element::boolean, "false" },
+    { "statistic-default-sample-count", Element::integer, "20" },
+    { "statistic-default-sample-age",   Element::integer, "0" }
 };
 
 /// @brief This table defines all option definition parameters.
@@ -125,7 +146,7 @@ const SimpleKeywords SimpleParser4::OPTION4_DEF_PARAMETERS = {
 /// definitions. This array lists default values for those option definitions.
 const SimpleDefaults SimpleParser4::OPTION4_DEF_DEFAULTS = {
     { "record-types", Element::string,  ""},
-    { "space",        Element::string,  "dhcp4"},
+    { "space",        Element::string,  "dhcp4"}, // DHCP4_OPTION_SPACE
     { "array",        Element::boolean, "false"},
     { "encapsulate",  Element::string,  "" }
 };
@@ -153,7 +174,7 @@ const SimpleKeywords SimpleParser4::OPTION4_PARAMETERS = {
 /// subnet, class or host reservations scopes. This array lists default values
 /// for those option-data declarations.
 const SimpleDefaults SimpleParser4::OPTION4_DEFAULTS = {
-    { "space",        Element::string,  "dhcp4"},
+    { "space",        Element::string,  "dhcp4"}, // DHCP4_OPTION_SPACE
     { "csv-format",   Element::boolean, "true"},
     { "always-send",  Element::boolean, "false"}
 };
@@ -164,35 +185,44 @@ const SimpleDefaults SimpleParser4::OPTION4_DEFAULTS = {
 /// list and map types for entries.
 /// Order follows subnet4_param rule in bison grammar.
 const SimpleKeywords SimpleParser4::SUBNET4_PARAMETERS = {
-    { "valid-lifetime",         Element::integer },
-    { "min-valid-lifetime",     Element::integer },
-    { "max-valid-lifetime",     Element::integer },
-    { "renew-timer",            Element::integer },
-    { "rebind-timer",           Element::integer },
-    { "option-data",            Element::list },
-    { "pools",                  Element::list },
-    { "subnet",                 Element::string },
-    { "interface",              Element::string },
-    { "id",                     Element::integer },
-    { "client-class",           Element::string },
-    { "require-client-classes", Element::list },
-    { "reservations",           Element::list },
-    { "reservation-mode",       Element::string },
-    { "relay",                  Element::map },
-    { "match-client-id",        Element::boolean },
-    { "authoritative",          Element::boolean },
-    { "next-server",            Element::string },
-    { "server-hostname",        Element::string },
-    { "boot-file-name",         Element::string },
-    { "4o6-interface",          Element::string },
-    { "4o6-interface-id",       Element::string },
-    { "4o6-subnet",             Element::string },
-    { "user-context",           Element::map },
-    { "comment",                Element::string },
-    { "calculate-tee-times",    Element::boolean },
-    { "t1-percent",             Element::real },
-    { "t2-percent",             Element::real },
-    { "metadata",               Element::map }
+    { "valid-lifetime",                 Element::integer },
+    { "min-valid-lifetime",             Element::integer },
+    { "max-valid-lifetime",             Element::integer },
+    { "renew-timer",                    Element::integer },
+    { "rebind-timer",                   Element::integer },
+    { "option-data",                    Element::list },
+    { "pools",                          Element::list },
+    { "subnet",                         Element::string },
+    { "interface",                      Element::string },
+    { "id",                             Element::integer },
+    { "client-class",                   Element::string },
+    { "require-client-classes",         Element::list },
+    { "reservations",                   Element::list },
+    { "reservation-mode",               Element::string },
+    { "relay",                          Element::map },
+    { "match-client-id",                Element::boolean },
+    { "authoritative",                  Element::boolean },
+    { "next-server",                    Element::string },
+    { "server-hostname",                Element::string },
+    { "boot-file-name",                 Element::string },
+    { "4o6-interface",                  Element::string },
+    { "4o6-interface-id",               Element::string },
+    { "4o6-subnet",                     Element::string },
+    { "user-context",                   Element::map },
+    { "comment",                        Element::string },
+    { "calculate-tee-times",            Element::boolean },
+    { "t1-percent",                     Element::real },
+    { "t2-percent",                     Element::real },
+    { "ddns-send-updates",              Element::boolean },
+    { "ddns-override-no-update",        Element::boolean },
+    { "ddns-override-client-update",    Element::boolean },
+    { "ddns-replace-client-name",       Element::string },
+    { "ddns-generated-prefix",          Element::string },
+    { "ddns-qualifying-suffix",         Element::string },
+    { "hostname-char-set",              Element::string },
+    { "hostname-char-replacement",      Element::string },
+    { "store-extended-info",            Element::boolean },
+    { "metadata",                       Element::map }
 };
 
 /// @brief This table defines default values for each IPv4 subnet.
@@ -234,23 +264,16 @@ const SimpleDefaults SimpleParser4::SHARED_SUBNET4_DEFAULTS = {
 /// This list is also used for inheriting from global to shared networks
 /// and from shared networks to subnets within it.
 const ParamsList SimpleParser4::INHERIT_TO_SUBNET4 = {
-    "boot-file-name",
-    "client-class",
-    "interface",
-    "match-client-id",
-    "authoritative",
-    "next-server",
     "rebind-timer",
     "relay",
     "renew-timer",
-    "reservation-mode",
-    "server-hostname",
     "valid-lifetime",
     "min-valid-lifetime",
     "max-valid-lifetime",
     "calculate-tee-times",
     "t1-percent",
-    "t2-percent"
+    "t2-percent",
+    "store-extended-info"
 };
 
 /// @brief This table defines all pool parameters.
@@ -274,30 +297,39 @@ const SimpleKeywords SimpleParser4::POOL4_PARAMETERS = {
 /// list and map types for entries.
 /// Order follows shared_network_param rule in bison grammar.
 const SimpleKeywords SimpleParser4::SHARED_NETWORK4_PARAMETERS = {
-    { "name",                   Element::string },
-    { "subnet4",                Element::list },
-    { "interface",              Element::string },
-    { "renew-timer",            Element::integer },
-    { "rebind-timer",           Element::integer },
-    { "option-data",            Element::list },
-    { "match-client-id",        Element::boolean },
-    { "authoritative",          Element::boolean },
-    { "next-server",            Element::string },
-    { "server-hostname",        Element::string },
-    { "boot-file-name",         Element::string },
-    { "relay",                  Element::map },
-    { "reservation-mode",       Element::string },
-    { "client-class",           Element::string },
-    { "require-client-classes", Element::list },
-    { "valid-lifetime",         Element::integer },
-    { "min-valid-lifetime",     Element::integer },
-    { "max-valid-lifetime",     Element::integer },
-    { "user-context",           Element::map },
-    { "comment",                Element::string },
-    { "calculate-tee-times",    Element::boolean },
-    { "t1-percent",             Element::real },
-    { "t2-percent",             Element::real },
-    { "metadata",               Element::map }
+    { "name",                           Element::string },
+    { "subnet4",                        Element::list },
+    { "interface",                      Element::string },
+    { "renew-timer",                    Element::integer },
+    { "rebind-timer",                   Element::integer },
+    { "option-data",                    Element::list },
+    { "match-client-id",                Element::boolean },
+    { "authoritative",                  Element::boolean },
+    { "next-server",                    Element::string },
+    { "server-hostname",                Element::string },
+    { "boot-file-name",                 Element::string },
+    { "relay",                          Element::map },
+    { "reservation-mode",               Element::string },
+    { "client-class",                   Element::string },
+    { "require-client-classes",         Element::list },
+    { "valid-lifetime",                 Element::integer },
+    { "min-valid-lifetime",             Element::integer },
+    { "max-valid-lifetime",             Element::integer },
+    { "user-context",                   Element::map },
+    { "comment",                        Element::string },
+    { "calculate-tee-times",            Element::boolean },
+    { "t1-percent",                     Element::real },
+    { "t2-percent",                     Element::real },
+    { "ddns-send-updates",              Element::boolean },
+    { "ddns-override-no-update",        Element::boolean },
+    { "ddns-override-client-update",    Element::boolean },
+    { "ddns-replace-client-name",       Element::string },
+    { "ddns-generated-prefix",          Element::string },
+    { "ddns-qualifying-suffix",         Element::string },
+    { "hostname-char-set",              Element::string },
+    { "hostname-char-replacement",      Element::string },
+    { "store-extended-info",            Element::boolean },
+    { "metadata",                       Element::map }
 };
 
 /// @brief This table defines default values for each IPv4 shared network.
@@ -315,7 +347,14 @@ const SimpleDefaults SimpleParser4::IFACE4_DEFAULTS = {
 const SimpleDefaults SimpleParser4::DHCP_QUEUE_CONTROL4_DEFAULTS = {
     { "enable-queue",   Element::boolean, "false"},
     { "queue-type",     Element::string,  "kea-ring4"},
-    { "capacity",       Element::integer, "500"}
+    { "capacity",       Element::integer, "64"}
+};
+
+/// @brief This table defines default values for multi-threading in DHCPv4.
+const SimpleDefaults SimpleParser4::DHCP_MULTI_THREADING4_DEFAULTS = {
+    { "enable-multi-threading", Element::boolean, "false" },
+    { "thread-pool-size",       Element::integer, "0" },
+    { "packet-queue-size",      Element::integer, "64" }
 };
 
 /// @brief This defines default values for sanity checking for DHCPv4.
@@ -389,6 +428,18 @@ size_t SimpleParser4::setAllDefaults(ElementPtr global) {
 
     cnt += setDefaults(mutable_cfg, DHCP_QUEUE_CONTROL4_DEFAULTS);
 
+    // Set the defaults for multi-threading.  If the element isn't there
+    // we'll add it.
+    ConstElementPtr multi_threading = global->get("multi-threading");
+    if (multi_threading) {
+        mutable_cfg = boost::const_pointer_cast<Element>(multi_threading);
+    } else {
+        mutable_cfg = Element::createMap();
+        global->set("multi-threading", mutable_cfg);
+    }
+
+    cnt += setDefaults(mutable_cfg, DHCP_MULTI_THREADING4_DEFAULTS);
+
     // Set the defaults for sanity-checks.  If the element isn't
     // there we'll add it.
     ConstElementPtr sanity_checks = global->get("sanity-checks");
@@ -436,12 +487,11 @@ size_t SimpleParser4::deriveParameters(ElementPtr global) {
                                                       INHERIT_TO_SUBNET4);
                 }
             }
-
         }
     }
 
     return (cnt);
 }
 
-};
-};
+}  // namespace dhcp
+}  // namespace isc
