@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -46,6 +46,17 @@ namespace test {
 bool testStatistics(const std::string& stat_name, const int64_t exp_value,
                     const SubnetID subnet_id = SUBNET_ID_UNUSED);
 
+/// @brief Get a value held by statistic manager.
+///
+/// This function may be used in some allocation tests and there's no
+/// single base class for it. @todo consider moving it src/lib/util.
+///
+/// @param stat_name Statistic name.
+/// @param subnet_id subnet_id of the desired subnet, if not zero.
+/// @return the value held by the statistic manager or zero.
+int64_t getStatistics(const std::string& stat_name,
+                      const SubnetID subnet_id = SUBNET_ID_UNUSED);
+
 /// @brief Allocation engine with some internal methods exposed
 class NakedAllocEngine : public AllocEngine {
 public:
@@ -63,6 +74,7 @@ public:
     using AllocEngine::Allocator;
     using AllocEngine::IterativeAllocator;
     using AllocEngine::getAllocator;
+    using AllocEngine::updateLease4ExtendedInfo;
 
     /// @brief IterativeAllocator with internal methods exposed
     class NakedIterativeAllocator: public AllocEngine::IterativeAllocator {
@@ -77,6 +89,23 @@ public:
         using AllocEngine::IterativeAllocator::increaseAddress;
         using AllocEngine::IterativeAllocator::increasePrefix;
     };
+
+    /// @brief Wrapper method for invoking AllocEngine4::updateLease4ExtendedInfo().
+    /// @param lease lease to update
+    /// @param ctx current packet processing context
+    void callUpdateLease4ExtendedInfo(const Lease4Ptr& lease,
+                                      AllocEngine::ClientContext4& ctx) const {
+        updateLease4ExtendedInfo(lease,ctx);
+    }
+
+    /// @brief Wrapper method for invoking AllocEngine6::updateLease6ExtendedInfo().
+    /// @param lease lease to update
+    /// @param ctx current packet processing context
+    void callUpdateLease6ExtendedInfo(const Lease6Ptr& lease,
+                                      AllocEngine::ClientContext6& ctx) const {
+        updateLease6ExtendedInfo(lease,ctx);
+    }
+
 };
 
 /// @brief Used in Allocation Engine tests for IPv6
@@ -439,6 +468,35 @@ public:
                       HWAddrPtr& hwaddr, const asiolink::IOAddress& addr,
                       uint8_t prefix_len);
 
+    /// @brief Utility function that decrements cltt of a persisted lease
+    ///
+    /// This function is used to simulate the passage of time by decrementing
+    /// the lease's cltt, currently by 1.  It fetches the desired lease from the
+    /// lease manager, decrements the cltt, then updates the lease in the lease
+    /// manager.  Next, it refetches the lease and verifies the update took place.
+    ///
+    /// @param[in][out] lease pointer reference to the lease to modify.  Upon
+    /// return it will point to the newly updated lease.
+    void
+    rollbackPersistedCltt(Lease6Ptr& lease) {
+        ASSERT_TRUE(lease) << "rollbackPersistedCltt lease is empty";
+
+        // Fetch it, so we can update it.
+        Lease6Ptr from_mgr = LeaseMgrFactory::instance().getLease6(lease->type_,
+                                                                   lease->addr_);
+        ASSERT_TRUE(from_mgr) << "rollbackPersistedCltt: lease not found?";
+
+        // Decrement cltt then update it in the manager.
+        --from_mgr->cltt_;
+        ASSERT_NO_THROW(LeaseMgrFactory::instance().updateLease6(from_mgr));
+
+        // Fetch it fresh.
+        lease = LeaseMgrFactory::instance().getLease6(lease->type_, lease->addr_);
+
+        // Make sure it stuck.
+        ASSERT_EQ(lease->cltt_, from_mgr->cltt_);
+    }
+
     virtual ~AllocEngine6Test() {
         factory_.destroy();
     }
