@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
 // Copyright (C) 2016-2017 Deutsche Telekom AG.
 //
 // Author: Andrei Pavel <andrei.pavel@qualitance.com>
@@ -190,8 +190,8 @@ TEST(CqlHostDataSource, OpenDatabase) {
     // Ensure we have the proper schema with no transient data.
     createCqlSchema();
 
-    // Check that host manager open the database opens correctly and tidy up.
-    //  If it fails, print the error message.
+    // Check that host manager opens the database correctly and tidy up.  If it
+    // fails, print the error message.
     try {
         HostMgr::create();
         EXPECT_NO_THROW(HostMgr::addBackend(validCqlConnectionString()));
@@ -203,7 +203,7 @@ TEST(CqlHostDataSource, OpenDatabase) {
                << "*** before the CQL tests will run correctly.\n";
     }
 
-    // Check that host manager open the database opens correctly with a longer
+    // Check that host manager opens the database correctly with a longer
     // timeout.  If it fails, print the error message.
     try {
         // CQL specifies the timeout values in ms, not seconds. Therefore
@@ -221,7 +221,7 @@ TEST(CqlHostDataSource, OpenDatabase) {
     }
 
     // Check that attempting to get an instance of the host data source when
-    // none is set throws an exception.
+    // none is set returns empty pointer.
     EXPECT_FALSE(HostMgr::instance().getHostDataSource());
 
     // Check that wrong specification of backend throws an exception.
@@ -332,6 +332,23 @@ TEST_F(CqlHostDataSourceTest, getAll6BySubnet) {
     testGetAll6();
 }
 
+// Verifies that host reservations with the same hostname can be retrieved
+TEST_F(CqlHostDataSourceTest, getAllbyHostname) {
+    testGetAllbyHostname();
+}
+
+// Verifies that IPv4 host reservations with the same hostname and in
+// the same subnet can be retrieved
+TEST_F(CqlHostDataSourceTest, getAllbyHostnameSubnet4) {
+    testGetAllbyHostnameSubnet4();
+}
+
+// Verifies that IPv6 host reservations with the same hostname and in
+// the same subnet can be retrieved
+TEST_F(CqlHostDataSourceTest, getAllbyHostnameSubnet6) {
+    testGetAllbyHostnameSubnet6();
+}
+
 // Verifies that IPv4 host reservations in the same subnet can be retrieved
 // by pages.
 TEST_F(CqlHostDataSourceTest, getPage4) {
