@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,18 +14,25 @@
 #include <http/http_messages.h>
 #include <http/response_json.h>
 #include <http/response_parser.h>
-#include <boost/bind.hpp>
+#include <util/multi_threading_mgr.h>
+#include <util/unlock_guard.h>
+
 #include <boost/enable_shared_from_this.hpp>
 #include <boost/weak_ptr.hpp>
-#include <array>
-#include <map>
-#include <queue>
 
+#include <atomic>
+#include <array>
+#include <functional>
 #include <iostream>
+#include <map>
+#include <mutex>
+#include <queue>
 
 using namespace isc;
 using namespace isc::asiolink;
-using namespace http;
+using namespace isc::http;
+using namespace isc::util;
+namespace ph = std::placeholders;
 
 namespace {
 
@@ -35,7 +42,7 @@ namespace {
 constexpr size_t MAX_LOGGED_MESSAGE_SIZE = 1024;
 
 /// @brief TCP socket callback function type.
-typedef boost::function<void(boost::system::error_code ec, size_t length)>
+typedef std::function<void(boost::system::error_code ec, size_t length)>
 SocketCallbackFunction;
 
 /// @brief Socket callback class required by the TCPSocket API.
@@ -92,19 +99,20 @@ typedef boost::shared_ptr<ConnectionPool> ConnectionPoolPtr;
 /// the new request is stored in the FIFO queue. The queued requests to the
 /// particular URL are sent to the server when the current transaction ends.
 ///
-/// The communication over the TCP socket is asynchronous. The caller is notified
-/// about the completion of the transaction via a callback that the caller supplies
-/// when initiating the transaction.
+/// The communication over the TCP socket is asynchronous. The caller is
+/// notified about the completion of the transaction via a callback that the
+/// caller supplies when initiating the transaction.
 class Connection : public boost::enable_shared_from_this<Connection> {
 public:
 
     /// @brief Constructor.
     ///
     /// @param io_service IO service to be used for the connection.
-    /// @param conn_pool Back pointer to the connection pool to which this connection
-    /// belongs.
+    /// @param conn_pool Back pointer to the connection pool to which this
+    /// connection belongs.
     /// @param url URL associated with this connection.
-    explicit Connection(IOService& io_service, const ConnectionPoolPtr& conn_pool,
+    explicit Connection(IOService& io_service,
+                        const ConnectionPoolPtr& conn_pool,
                         const Url& url);
 
     /// @brief Destructor.
@@ -115,19 +123,21 @@ public:
     /// This method expects that all pointers provided as argument are non-null.
     ///
     /// @param request Pointer to the request to be sent to the server.
-    /// @param response Pointer to the object into which the response is stored. The
-    /// caller should create a response object of the type which matches the content
-    /// type expected by the caller, e.g. HttpResponseJson when JSON content type
-    /// is expected to be received.
+    /// @param response Pointer to the object into which the response is stored.
+    /// The caller should create a response object of the type which matches the
+    /// content type expected by the caller, e.g. HttpResponseJson when JSON
+    /// content type is expected to be received.
     /// @param request_timeout Request timeout in milliseconds.
     /// @param callback Pointer to the callback function to be invoked when the
     /// transaction completes.
-    /// @param connect_callback Pointer to the callback function to be invoked when
-    /// the client connects to the server.
-    /// @param close_callback Pointer to the callback function to be invoked when
-    /// the client closes the socket to the server.
-    void doTransaction(const HttpRequestPtr& request, const HttpResponsePtr& response,
-                       const long request_timeout, const HttpClient::RequestHandler& callback,
+    /// @param connect_callback Pointer to the callback function to be invoked
+    /// when the client connects to the server.
+    /// @param close_callback Pointer to the callback function to be invoked
+    /// when the client closes the socket to the server.
+    void doTransaction(const HttpRequestPtr& request,
+                       const HttpResponsePtr& response,
+                       const long request_timeout,
+                       const HttpClient::RequestHandler& callback,
                        const HttpClient::ConnectHandler& connect_callback,
                        const HttpClient::CloseHandler& close_callback);
 
@@ -165,8 +175,59 @@ public:
 
 private:
 
+    /// @brief Starts new asynchronous transaction (HTTP request and response).
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// This method expects that all pointers provided as argument are non-null.
+    ///
+    /// @param request Pointer to the request to be sent to the server.
+    /// @param response Pointer to the object into which the response is stored.
+    /// The caller should create a response object of the type which matches the
+    /// content type expected by the caller, e.g. HttpResponseJson when JSON
+    /// content type is expected to be received.
+    /// @param request_timeout Request timeout in milliseconds.
+    /// @param callback Pointer to the callback function to be invoked when the
+    /// transaction completes.
+    /// @param connect_callback Pointer to the callback function to be invoked
+    /// when the client connects to the server.
+    /// @param close_callback Pointer to the callback function to be invoked
+    /// when the client closes the socket to the server.
+    void doTransactionInternal(const HttpRequestPtr& request,
+                               const HttpResponsePtr& response,
+                               const long request_timeout,
+                               const HttpClient::RequestHandler& callback,
+                               const HttpClient::ConnectHandler& connect_callback,
+                               const HttpClient::CloseHandler& close_callback);
+
+    /// @brief Closes the socket and cancels the request timer.
+    ///
+    /// Should be called in a thread safe context.
+    void closeInternal();
+
+    /// @brief Checks and logs if premature transaction timeout is suspected.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// There are cases when the premature timeout occurs, e.g. as a result of
+    /// moving system clock, during the transaction. In such case, the
+    /// @c terminate function is called which resets the transaction state but
+    /// the transaction handlers may be already waiting for the execution.
+    /// Each such handler should call this function to check if the transaction
+    /// it is participating in is still alive. If it is not, it should simply
+    /// return. This method also logs such situation.
+    ///
+    /// @param transid identifier of the transaction for which the handler
+    /// is being invoked. It is compared against the current transaction
+    /// id for this connection.
+    ///
+    /// @return true if the premature timeout is suspected, false otherwise.
+    bool checkPrematureTimeoutInternal(const uint64_t transid);
+
     /// @brief Resets the state of the object.
     ///
+    /// Should be called in a thread safe context.
+    ///
     /// In particular, it removes instances of objects provided for the previous
     /// transaction by a caller. It doesn't close the socket, though.
     void resetState();
@@ -183,6 +244,38 @@ private:
     void terminate(const boost::system::error_code& ec,
                    const std::string& parsing_error = "");
 
+    /// @brief Performs tasks required after receiving a response or after an
+    /// error.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// This method triggers user's callback, resets the state of the connection
+    /// and initiates next transaction if there is any transaction queued for the
+    /// URL associated with this connection.
+    ///
+    /// @param ec Error code received as a result of the IO operation.
+    /// @param parsing_error Message parsing error.
+    void terminateInternal(const boost::system::error_code& ec,
+                           const std::string& parsing_error = "");
+
+    /// @brief Run parser and check if more data is needed.
+    ///
+    /// @param ec Error code received as a result of the IO operation.
+    /// @param length Number of bytes received.
+    ///
+    /// @return true if more data is needed, false otherwise.
+    bool runParser(const boost::system::error_code& ec, size_t length);
+
+    /// @brief Run parser and check if more data is needed.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// @param ec Error code received as a result of the IO operation.
+    /// @param length Number of bytes received.
+    ///
+    /// @return true if more data is needed, false otherwise.
+    bool runParserInternal(const boost::system::error_code& ec, size_t length);
+
     /// @brief This method schedules timer or reschedules existing timer.
     ///
     /// @param request_timeout New timer interval in milliseconds.
@@ -289,6 +382,12 @@ private:
 
     /// @brief User supplied callback.
     HttpClient::CloseHandler close_callback_;
+
+    /// @brief Flag to indicate that a transaction is running.
+    std::atomic<bool> started_;
+
+    /// @brief Mutex to protect the internal state.
+    std::mutex mutex_;
 };
 
 /// @brief Shared pointer to the connection.
@@ -309,7 +408,7 @@ public:
     /// @param io_service Reference to the IO service to be used by the
     /// connections.
     explicit ConnectionPool(IOService& io_service)
-        : io_service_(io_service), conns_(), queue_() {
+        : io_service_(io_service), conns_(), queue_(), mutex_() {
     }
 
     /// @brief Destructor.
@@ -319,52 +418,21 @@ public:
         closeAll();
     }
 
-    /// @brief Returns next queued request for the given URL.
+    /// @brief Process next queued request for the given URL.
     ///
-    /// @param url URL for which next queued request should be retrieved.
-    /// @param [out] request Pointer to the queued request.
-    /// @param [out] response Pointer to the object into which response should
-    /// be stored.
-    /// @param request_timeout Requested timeout for the transaction.
-    /// @param callback Pointer to the user callback for this request.
-    /// @param connect_callback Pointer to the user callback invoked when
-    /// the client connects to the server.
-    /// @param close_callback Pointer to the user callback invoked when
-    /// the client closes the connection to the server.
-    ///
-    /// @return true if the request for the given URL has been retrieved,
-    /// false if there are no more requests queued for this URL.
-    bool getNextRequest(const Url& url,
-                        HttpRequestPtr& request,
-                        HttpResponsePtr& response,
-                        long& request_timeout,
-                        HttpClient::RequestHandler& callback,
-                        HttpClient::ConnectHandler& connect_callback,
-                        HttpClient::CloseHandler& close_callback) {
-        // Check if there is a queue for this URL. If there is no queue, there
-        // is no request queued either.
-        auto it = queue_.find(url);
-        if (it != queue_.end()) {
-            // If the queue is non empty, we take the oldest request.
-            if (!it->second.empty()) {
-                RequestDescriptor desc = it->second.front();
-                it->second.pop();
-                request = desc.request_;
-                response = desc.response_;
-                request_timeout = desc.request_timeout_,
-                callback = desc.callback_;
-                connect_callback = desc.connect_callback_;
-                close_callback = desc.close_callback_;
-                return (true);
-            }
+    /// @param url URL for which next queued request should be processed.
+    void processNextRequest(const Url& url) {
+        if (MultiThreadingMgr::instance().getMode()) {
+            std::lock_guard<std::mutex> lk(mutex_);
+            return (processNextRequestInternal(url));
+        } else {
+            return (processNextRequestInternal(url));
         }
-
-        return (false);
     }
 
     /// @brief Queue next request for sending to the server.
     ///
-    /// A new transaction is started immediatelly, if there is no other request
+    /// A new transaction is started immediately, if there is no other request
     /// in progress for the given URL. Otherwise, the request is queued.
     ///
     /// @param url Destination where the request should be sent.
@@ -386,24 +454,130 @@ public:
                       const HttpClient::RequestHandler& request_callback,
                       const HttpClient::ConnectHandler& connect_callback,
                       const HttpClient::CloseHandler& close_callback) {
+        if (MultiThreadingMgr::instance().getMode()) {
+            std::lock_guard<std::mutex> lk(mutex_);
+            return (queueRequestInternal(url, request, response,
+                                         request_timeout, request_callback,
+                                         connect_callback, close_callback));
+        } else {
+            return (queueRequestInternal(url, request, response,
+                                         request_timeout, request_callback,
+                                         connect_callback, close_callback));
+        }
+    }
+
+    /// @brief Closes connection and removes associated information from the
+    /// connection pool.
+    ///
+    /// @param url URL for which connection should be closed.
+    void closeConnection(const Url& url) {
+        if (MultiThreadingMgr::instance().getMode()) {
+            std::lock_guard<std::mutex> lk(mutex_);
+            closeConnectionInternal(url);
+        } else {
+            closeConnectionInternal(url);
+        }
+    }
+
+    /// @brief Closes all connections and removes associated information from
+    /// the connection pool.
+    void closeAll() {
+        if (MultiThreadingMgr::instance().getMode()) {
+            std::lock_guard<std::mutex> lk(mutex_);
+            closeAllInternal();
+        } else {
+            closeAllInternal();
+        }
+    }
+
+    /// @brief Closes a connection if it has an out-of-band socket event
+    ///
+    /// If the pool contains a connection using the given socket and that
+    /// connection is currently in a transaction the method returns as this
+    /// indicates a normal ready event.  If the connection is not in an
+    /// ongoing transaction, then the connection is closed.
+    ///
+    /// This is method is intended to be used to detect and clean up then
+    /// sockets that are marked ready outside of transactions. The most common
+    /// case is the other end of the socket being closed.
+    ///
+    /// @param socket_fd socket descriptor to check
+    void closeIfOutOfBand(int socket_fd) {
+        if (MultiThreadingMgr::instance().getMode()) {
+            std::lock_guard<std::mutex> lk(mutex_);
+            closeIfOutOfBandInternal(socket_fd);
+        } else {
+            closeIfOutOfBandInternal(socket_fd);
+        }
+    }
+
+private:
+
+    /// @brief Process next queued request for the given URL.
+    ///
+    /// This method should be called in a thread safe context.
+    ///
+    /// @param url URL for which next queued request should be retrieved.
+    void processNextRequestInternal(const Url& url) {
+        // Check if there is a queue for this URL. If there is no queue, there
+        // is no request queued either.
+        auto it = queue_.find(url);
+        if (it != queue_.end()) {
+            // If the queue is non empty, we take the oldest request.
+            if (!it->second.empty()) {
+                RequestDescriptor desc = it->second.front();
+                it->second.pop();
+                desc.conn_->doTransaction(desc.request_, desc.response_,
+                                          desc.request_timeout_, desc.callback_,
+                                          desc.connect_callback_,
+                                          desc.close_callback_);
+            }
+        }
+    }
+
+    /// @brief Queue next request for sending to the server.
+    ///
+    /// A new transaction is started immediately, if there is no other request
+    /// in progress for the given URL. Otherwise, the request is queued.
+    ///
+    /// This method should be called in a thread safe context.
+    ///
+    /// @param url Destination where the request should be sent.
+    /// @param request Pointer to the request to be sent to the server.
+    /// @param response Pointer to the object into which the response should be
+    /// stored.
+    /// @param request_timeout Requested timeout for the transaction in
+    /// milliseconds.
+    /// @param request_callback Pointer to the user callback to be invoked when the
+    /// transaction ends.
+    /// @param connect_callback Pointer to the user callback to be invoked when the
+    /// client connects to the server.
+    /// @param close_callback Pointer to the user callback to be invoked when the
+    /// client closes the connection to the server.
+    void queueRequestInternal(const Url& url,
+                              const HttpRequestPtr& request,
+                              const HttpResponsePtr& response,
+                              const long request_timeout,
+                              const HttpClient::RequestHandler& request_callback,
+                              const HttpClient::ConnectHandler& connect_callback,
+                              const HttpClient::CloseHandler& close_callback) {
         auto it = conns_.find(url);
         if (it != conns_.end()) {
             ConnectionPtr conn = it->second;
             // There is a connection for this URL already. Check if it is idle.
             if (conn->isTransactionOngoing()) {
                 // Connection is busy, so let's queue the request.
-                queue_[url].push(RequestDescriptor(request, response,
+                queue_[url].push(RequestDescriptor(conn, request, response,
                                                    request_timeout,
                                                    request_callback,
                                                    connect_callback,
                                                    close_callback));
-
             } else {
                 // Connection is idle, so we can start the transaction.
                 conn->doTransaction(request, response, request_timeout,
-                                    request_callback, connect_callback, close_callback);
+                                    request_callback, connect_callback,
+                                    close_callback);
             }
-
         } else {
             // There is no connection with this destination yet. Let's create
             // it and start the transaction.
@@ -418,8 +592,10 @@ public:
     /// @brief Closes connection and removes associated information from the
     /// connection pool.
     ///
-    /// @param url URL for which connection shuld be closed.
-    void closeConnection(const Url& url) {
+    /// This method should be called in a thread safe context.
+    ///
+    /// @param url URL for which connection should be closed.
+    void closeConnectionInternal(const Url& url) {
         // Close connection for the specified URL.
         auto conns_it = conns_.find(url);
         if (conns_it != conns_.end()) {
@@ -436,7 +612,9 @@ public:
 
     /// @brief Closes all connections and removes associated information from
     /// the connection pool.
-    void closeAll() {
+    ///
+    /// This method should be called in a thread safe context.
+    void closeAllInternal() {
         for (auto conns_it = conns_.begin(); conns_it != conns_.end();
              ++conns_it) {
             conns_it->second->close();
@@ -446,7 +624,7 @@ public:
         queue_.clear();
     }
 
-    /// @brief Closes a connection if it has an out-of-bandwidth socket event
+    /// @brief Closes a connection if it has an out-of-band socket event
     ///
     /// If the pool contains a connection using the given socket and that
     /// connection is currently in a transaction the method returns as this
@@ -454,11 +632,13 @@ public:
     /// ongoing transaction, then the connection is closed.
     ///
     /// This is method is intended to be used to detect and clean up then
-    /// sockets that are marked ready outside of transactions. The most comman
+    /// sockets that are marked ready outside of transactions. The most common
     /// case is the other end of the socket being closed.
     ///
+    /// This method should be called in a thread safe context.
+    ///
     /// @param socket_fd socket descriptor to check
-    void closeIfOutOfBandwidth(int socket_fd) {
+    void closeIfOutOfBandInternal(int socket_fd) {
         // First we look for a connection with the socket.
         for (auto conns_it = conns_.begin(); conns_it != conns_.end();
              ++conns_it) {
@@ -474,7 +654,7 @@ public:
             }
 
             // Socket has no transaction, so any ready event is
-            // out-of-bandwidth (other end probably closed), so
+            // out-of-band (other end probably closed), so
             // let's close it.  Note we do not remove any queued
             // requests, as this might somehow be occurring in
             // between them.
@@ -484,8 +664,6 @@ public:
         }
     }
 
-private:
-
     /// @brief Holds reference to the IO service.
     IOService& io_service_;
 
@@ -497,6 +675,7 @@ private:
     struct RequestDescriptor {
         /// @brief Constructor.
         ///
+        /// @param conn Pointer to the connection.
         /// @param request Pointer to the request to be sent.
         /// @param response Pointer to the object into which the response will
         /// be stored.
@@ -506,27 +685,34 @@ private:
         /// when the client connects to the server.
         /// @param close_callback pointer to the user callback to be invoked
         /// when the client closes the connection to the server.
-        RequestDescriptor(const HttpRequestPtr& request,
+        RequestDescriptor(const ConnectionPtr& conn,
+                          const HttpRequestPtr& request,
                           const HttpResponsePtr& response,
-                          const long request_timeout,
+                          const long& request_timeout,
                           const HttpClient::RequestHandler& callback,
                           const HttpClient::ConnectHandler& connect_callback,
                           const HttpClient::CloseHandler& close_callback)
-            : request_(request), response_(response),
-              request_timeout_(request_timeout),
-              callback_(callback),
+            : conn_(conn), request_(request), response_(response),
+              request_timeout_(request_timeout), callback_(callback),
               connect_callback_(connect_callback),
               close_callback_(close_callback) {
         }
 
+        /// @brief Holds the connection.
+        ConnectionPtr conn_;
+
         /// @brief Holds pointer to the request.
         HttpRequestPtr request_;
+
         /// @brief Holds pointer to the response.
         HttpResponsePtr response_;
+
         /// @brief Holds requested timeout value.
         long request_timeout_;
+
         /// @brief Holds pointer to the user callback.
         HttpClient::RequestHandler callback_;
+
         /// @brief Holds pointer to the user callback for connect.
         HttpClient::ConnectHandler connect_callback_;
 
@@ -536,6 +722,9 @@ private:
 
     /// @brief Holds the queue of requests for different URLs.
     std::map<Url, std::queue<RequestDescriptor> > queue_;
+
+    /// @brief Mutex to protect the internal state.
+    std::mutex mutex_;
 };
 
 Connection::Connection(IOService& io_service,
@@ -543,7 +732,8 @@ Connection::Connection(IOService& io_service,
                        const Url& url)
     : conn_pool_(conn_pool), url_(url), socket_(io_service), timer_(io_service),
       current_request_(), current_response_(), parser_(), current_callback_(),
-      buf_(), input_buf_(), current_transid_(0), close_callback_() {
+      buf_(), input_buf_(), current_transid_(0), close_callback_(),
+      started_(false) {
 }
 
 Connection::~Connection() {
@@ -552,13 +742,13 @@ Connection::~Connection() {
 
 void
 Connection::resetState() {
+    started_ = false;
     current_request_.reset();
     current_response_.reset();
     parser_.reset();
     current_callback_ = HttpClient::RequestHandler();
 }
 
-
 void
 Connection::closeCallback(const bool clear) {
     if (close_callback_) {
@@ -574,7 +764,6 @@ Connection::closeCallback(const bool clear) {
     }
 }
 
-
 void
 Connection::doTransaction(const HttpRequestPtr& request,
                           const HttpResponsePtr& response,
@@ -582,7 +771,25 @@ Connection::doTransaction(const HttpRequestPtr& request,
                           const HttpClient::RequestHandler& callback,
                           const HttpClient::ConnectHandler& connect_callback,
                           const HttpClient::CloseHandler& close_callback) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        doTransactionInternal(request, response, request_timeout,
+                              callback, connect_callback, close_callback);
+    } else {
+        doTransactionInternal(request, response, request_timeout,
+                              callback, connect_callback, close_callback);
+    }
+}
+
+void
+Connection::doTransactionInternal(const HttpRequestPtr& request,
+                                  const HttpResponsePtr& response,
+                                  const long request_timeout,
+                                  const HttpClient::RequestHandler& callback,
+                                  const HttpClient::ConnectHandler& connect_callback,
+                                  const HttpClient::CloseHandler& close_callback) {
     try {
+        started_ = true;
         current_request_ = request;
         current_response_ = response;
         parser_.reset(new HttpResponseParser(*current_response_));
@@ -625,8 +832,9 @@ Connection::doTransaction(const HttpRequestPtr& request,
         /// over for names.
         TCPEndpoint endpoint(url_.getStrippedHostname(),
                              static_cast<unsigned short>(url_.getPort()));
-        SocketCallback socket_cb(boost::bind(&Connection::connectCallback, shared_from_this(),
-                                             connect_callback, current_transid_, _1));
+        SocketCallback socket_cb(std::bind(&Connection::connectCallback, shared_from_this(),
+                                           connect_callback, current_transid_,
+                                           ph::_1));
 
         // Establish new connection or use existing connection.
         socket_.open(&endpoint, socket_cb);
@@ -639,17 +847,28 @@ Connection::doTransaction(const HttpRequestPtr& request,
 
 void
 Connection::close() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        return (closeInternal());
+    } else {
+        return (closeInternal());
+    }
+}
+
+void
+Connection::closeInternal() {
     // Pass in true to discard the callback.
     closeCallback(true);
 
     timer_.cancel();
     socket_.close();
+
     resetState();
 }
 
 bool
 Connection::isTransactionOngoing() const {
-    return (static_cast<bool>(current_request_));
+    return (started_);
 }
 
 bool
@@ -659,6 +878,16 @@ Connection::isMySocket(int socket_fd) const {
 
 bool
 Connection::checkPrematureTimeout(const uint64_t transid) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        return (checkPrematureTimeoutInternal(transid));
+    } else {
+        return (checkPrematureTimeoutInternal(transid));
+    }
+}
+
+bool
+Connection::checkPrematureTimeoutInternal(const uint64_t transid) {
     // If there is no transaction but the handlers are invoked it means
     // that the last transaction in the queue timed out prematurely.
     // Also, if there is a transaction in progress but the ID of that
@@ -674,7 +903,17 @@ Connection::checkPrematureTimeout(const uint64_t transid) {
 void
 Connection::terminate(const boost::system::error_code& ec,
                       const std::string& parsing_error) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        terminateInternal(ec, parsing_error);
+    } else {
+        terminateInternal(ec, parsing_error);
+    }
+}
 
+void
+Connection::terminateInternal(const boost::system::error_code& ec,
+                              const std::string& parsing_error) {
     HttpResponsePtr response;
 
     if (isTransactionOngoing()) {
@@ -692,11 +931,13 @@ Connection::terminate(const boost::system::error_code& ec,
             LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_BASIC_DATA,
                       HTTP_SERVER_RESPONSE_RECEIVED_DETAILS)
                 .arg(url_.toText())
-                .arg((parser_ ? parser_->getBufferAsString(MAX_LOGGED_MESSAGE_SIZE)
-                      : "[HttpResponseParser is null]"));
+                .arg(parser_ ?
+                     parser_->getBufferAsString(MAX_LOGGED_MESSAGE_SIZE) :
+                     "[HttpResponseParser is null]");
 
         } else {
-            std::string err = parsing_error.empty() ? ec.message() : parsing_error;
+            std::string err = parsing_error.empty() ? ec.message() :
+                                                      parsing_error;
 
             LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_BASIC,
                       HTTP_BAD_SERVER_RESPONSE_RECEIVED)
@@ -709,23 +950,28 @@ Connection::terminate(const boost::system::error_code& ec,
                 LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_BASIC_DATA,
                           HTTP_BAD_SERVER_RESPONSE_RECEIVED_DETAILS)
                     .arg(url_.toText())
-                    .arg((parser_ ? parser_->getBufferAsString()
-                          : "[HttpResponseParser is null]"));
+                    .arg(parser_ ?
+                         parser_->getBufferAsString(MAX_LOGGED_MESSAGE_SIZE) :
+                         "[HttpResponseParser is null]");
             }
         }
 
         try {
             // The callback should take care of its own exceptions but one
             // never knows.
-            current_callback_(ec, response, parsing_error);
-
+            if (MultiThreadingMgr::instance().getMode()) {
+                UnlockGuard<std::mutex> lock(mutex_);
+                current_callback_(ec, response, parsing_error);
+            } else {
+                current_callback_(ec, response, parsing_error);
+            }
         } catch (...) {
         }
 
-        // If we're not requesting connection persistence, we should close the socket.
-        // We're going to reconnect for the next transaction.
-        if (!current_request_->isPersistent()) {
-            close();
+        // If we're not requesting connection persistence or the connection has timed out,
+        // we should close the socket. We're going to reconnect for the next transaction.
+        if (!current_request_->isPersistent() || (ec == boost::asio::error::timed_out)) {
+            closeInternal();
         }
 
         resetState();
@@ -733,31 +979,32 @@ Connection::terminate(const boost::system::error_code& ec,
 
     // Check if there are any requests queued for this connection and start
     // another transaction if there is at least one.
-    HttpRequestPtr request;
-    long request_timeout;
-    HttpClient::RequestHandler callback;
-    HttpClient::ConnectHandler connect_callback;
-    HttpClient::CloseHandler close_callback;
     ConnectionPoolPtr conn_pool = conn_pool_.lock();
-    if (conn_pool && conn_pool->getNextRequest(url_, request, response, request_timeout,
-                                               callback, connect_callback, close_callback)) {
-        doTransaction(request, response, request_timeout, callback,
-                      connect_callback, close_callback);
+    if (conn_pool) {
+        if (MultiThreadingMgr::instance().getMode()) {
+            UnlockGuard<std::mutex> lock(mutex_);
+            conn_pool->processNextRequest(url_);
+        } else {
+            conn_pool->processNextRequest(url_);
+        }
     }
 }
 
 void
 Connection::scheduleTimer(const long request_timeout) {
     if (request_timeout > 0) {
-        timer_.setup(boost::bind(&Connection::timerCallback, this), request_timeout,
+        timer_.setup(std::bind(&Connection::timerCallback, this), request_timeout,
                      IntervalTimer::ONE_SHOT);
     }
 }
 
 void
 Connection::doSend(const uint64_t transid) {
-    SocketCallback socket_cb(boost::bind(&Connection::sendCallback, shared_from_this(),
-                                         transid, _1, _2));
+    SocketCallback socket_cb(std::bind(&Connection::sendCallback,
+                                       shared_from_this(),
+                                       transid,
+                                       ph::_1,
+                                       ph::_2));
     try {
         socket_.asyncSend(&buf_[0], buf_.size(), socket_cb);
 
@@ -769,8 +1016,11 @@ Connection::doSend(const uint64_t transid) {
 void
 Connection::doReceive(const uint64_t transid) {
     TCPEndpoint endpoint;
-    SocketCallback socket_cb(boost::bind(&Connection::receiveCallback, shared_from_this(),
-                                         transid, _1, _2));
+    SocketCallback socket_cb(std::bind(&Connection::receiveCallback,
+                                       shared_from_this(),
+                                       transid,
+                                       ph::_1,
+                                       ph::_2));
 
     try {
         socket_.asyncReceive(static_cast<void*>(input_buf_.data()), input_buf_.size(), 0,
@@ -816,7 +1066,8 @@ Connection::connectCallback(HttpClient::ConnectHandler connect_callback,
 }
 
 void
-Connection::sendCallback(const uint64_t transid, const boost::system::error_code& ec,
+Connection::sendCallback(const uint64_t transid,
+                         const boost::system::error_code& ec,
                          size_t length) {
     if (checkPrematureTimeout(transid)) {
         return;
@@ -859,7 +1110,8 @@ Connection::sendCallback(const uint64_t transid, const boost::system::error_code
 }
 
 void
-Connection::receiveCallback(const uint64_t transid, const boost::system::error_code& ec,
+Connection::receiveCallback(const uint64_t transid,
+                            const boost::system::error_code& ec,
                             size_t length) {
     if (checkPrematureTimeout(transid)) {
         return;
@@ -886,6 +1138,24 @@ Connection::receiveCallback(const uint64_t transid, const boost::system::error_c
     // Receiving is in progress, so push back the timeout.
     scheduleTimer(timer_.getInterval());
 
+    if (runParser(ec, length)) {
+        doReceive(transid);
+    }
+}
+
+bool
+Connection::runParser(const boost::system::error_code& ec, size_t length) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        return (runParserInternal(ec, length));
+    } else {
+        return (runParserInternal(ec, length));
+    }
+}
+
+bool
+Connection::runParserInternal(const boost::system::error_code& ec,
+                              size_t length) {
     // If we have received any data, let's feed the parser with it.
     if (length != 0) {
         parser_->postBuffer(static_cast<void*>(input_buf_.data()), length);
@@ -894,25 +1164,27 @@ Connection::receiveCallback(const uint64_t transid, const boost::system::error_c
 
     // If the parser still needs data, let's schedule another receive.
     if (parser_->needData()) {
-        doReceive(transid);
+        return (true);
 
     } else if (parser_->httpParseOk()) {
         // No more data needed and parsing has been successful so far. Let's
         // try to finalize the response parsing.
         try {
             current_response_->finalize();
-            terminate(ec);
+            terminateInternal(ec);
 
         } catch (const std::exception& ex) {
             // If there is an error here, we need to return the error message.
-            terminate(ec, ex.what());
+            terminateInternal(ec, ex.what());
         }
 
     } else {
-        // Parsing was unsuccessul. Let's pass the error message held in the
+        // Parsing was unsuccessful. Let's pass the error message held in the
         // parser.
-        terminate(ec, parser_->getErrorMessage());
+        terminateInternal(ec, parser_->getErrorMessage());
     }
+
+    return (false);
 }
 
 void
@@ -974,8 +1246,8 @@ HttpClient::asyncSendRequest(const Url& url, const HttpRequestPtr& request,
 }
 
 void
-HttpClient::closeIfOutOfBandwidth(int socket_fd)  {
-    return (impl_->conn_pool_->closeIfOutOfBandwidth(socket_fd));
+HttpClient::closeIfOutOfBand(int socket_fd)  {
+    return (impl_->conn_pool_->closeIfOutOfBand(socket_fd));
 }
 
 void
