@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -186,7 +186,7 @@ LoggerManager::readLocalMessageFile(const char* file) {
                 logger.warn(LOG_NO_SUCH_MESSAGE).arg(message_id);
         }
     }
-    catch (MessageException& e) {
+    catch (const MessageException& e) {
         MessageID ident = e.id();
         vector<string> args = e.arguments();
 
@@ -206,9 +206,9 @@ LoggerManager::reset() {
     LoggerManagerImpl::reset(initSeverity(), initDebugLevel());
 }
 
-isc::util::thread::Mutex&
+std::mutex&
 LoggerManager::getMutex() {
-    static isc::util::thread::Mutex mutex;
+    static std::mutex mutex;
 
     return (mutex);
 }
